## clustglm_fun.R

## Uses glm and model.matrix

## Bernoulli, binomial, Poisson and negative binomial distributions

## Initiates EM by random choice of parameters, not allocations.

## Model fitting function is clustglm.fn


## Function names:
## ---------------

## xlogx.fn
## log.fn
## logit.fn
## expit.fn

## bernoulli.logpdf
## binomial.logpdf
## poisson.logpdf
## NB.logpdf

## AIC.fn
## AICc.fn
## BIC.fn

## clustglm

## E.step.fn
## M.step.bi.fn
## M.step.rows.fn
## M.step.cols.fn

## dkm.fn
## randpart.fn
## mat2longdf

## ICtable.fn

## ICplot.fn
## ordplot.fn
## profileplot.fn
## biplot.fn



#####################################################

## xlogx function:
## ---------------

## Function for x.log(x) with zero if x=0 (or if computer puts x
## just below zero). 
## Argument may be a scalar, vector or matrix.

xlogx.fn <- function(x){
    xlx <- x
    xlx[xlx <= 0] <- 0
    xlx[xlx > 0]  <- x[x>0] * log(x[x>0])
    return(xlx)
}


## log function:
## -------------

## If p is really close to 0, set log to some min.

log.fn <- function(x){
    ## Replace values if too near 0:
    x[x<0.000001] <- 0.000001
    ## Output object:
    return(log(x))
}


## logit function:
## ---------------

## If p is really close to 0 or 1, set logit to some max or min.

logit.fn <- function(x){
    ## Replace values if too near 0 or 1:
    x[x<0.000001] <- 0.000001
    x[x>0.999999] <- 0.999999
    ## Output vector:
    return(log(x/(1-x)))
}

## expit function:
## ---------------

## Inverse of logit function, acts on a logit vector.

expit.fn <- function(x)
    return(1/(1+exp(-x)))


## identity function:
## ------------------

id.fn <- function(x)
    return(x)


## logpdf functions:
## -----------------

bernoulli.logpdf <- function(theta.ar, y.ar){
    y.ar*log.fn(theta.ar) + (1-y.ar)*log.fn(1-theta.ar)
}

binomial.logpdf <- function(theta.ar, nt.ar, y.ar){
    lgamma(nt.ar+1) - lgamma(y.ar+1) - lgamma(nt.ar-y.ar+1) +
        y.ar*log.fn(theta.ar) + (nt.ar-y.ar)*log.fn(1-theta.ar)
}

poisson.logpdf <- function(mu.ar,y.ar){
    -mu.ar + y.ar * log.fn(mu.ar) - lgamma(y.ar+1)
}

NB.logpdf <- function(mu.ar,k.ar,y.ar){
    lpdf <- lgamma(y.ar + k.ar) - lgamma(y.ar+1) -
        lgamma(k.ar) + y.ar*log.fn(mu.ar) +
            k.ar*log.fn(k.ar) -
                (y.ar+k.ar)*log.fn(mu.ar+k.ar)
    return(lpdf)
}

gaussian.logpdf <- function(mu.ar,var.sc,y.ar){
    -log(2*pi*var.sc) -0.5*(y.ar-mu.ar)^2/var.sc
}


## IC functions:
## -------------

## y = LL, x = npar

AIC.fn <- function(x,y,n,p)
    2*x-2*y

AICc.fn <- function(x,y,n,p)
    2*x + (2*(x+1)*(x+2))/(n*p-x-2)-2*y

BIC.fn <- function(x,y,n,p)
    x*log.fn(n*p)-2*y



######################################################

## Model fitting function:
## -----------------------

clustglm <- function(glmformula, glmfamily = "gaussian", data, weights, subset,
                     na.action, start = NULL, etastart, mustart, offset,
                     control = list(...), model = TRUE, method = "glm.fit",
                     x = FALSE, y = TRUE, contrasts = NULL, std, 
                     clustfact = NULL, clustfactnames = NULL,
                     numbersclusters = NULL,  clustformula = NULL,
                     cov.interaction = F, ntrials = 0, 
                     ppr.start = NULL, ppc.start = NULL,
                     RGsplit = NULL, CGsplit = NULL,
                     startEMcycles = 3, kstarts = 100, 
                     Hclust = TRUE, Diana = TRUE,
                     randstarts = 0, EMcycles = 100, 
                     EMstoppingpar = 1e-04, ...){
    
    ## Stop if binomial but ntrials not given:
    if ((glmfamily=="binomial")&(ntrials==0))
        stop("For binomial distribution, please specify the number of trials")
    
    ## Need to read in n and p
    ## Assume data came from a matrix with labels ROW and COL.
    
    n <- length(levels(data$ROW))
    p <- length(levels(data$COL))
    Nobs <- n*p
    
    
    ## Run the glm (no clustering):
    ## ----------------------------
    
    std.glm <- glm(formula = glmformula,family = glmfamily,data = data)
    std.glm
    
    print("Std.glm fitted")
    
    glm.npar <- length(std.glm$coefficients)
    
    std.glm.means <- matrix(fitted(std.glm),n,p)
    dimnames(std.glm.means) <- dimnames(y.mat)
    fits.m <- std.glm.means
    var.s <- 0
    cdf.data.m <- matrix(NA,n,p)
    rqresid.m <- matrix(NA,n,p)
    dimnames(rqresid.m) <- dimnames(y.mat)
    
    for(i in 1:n) for(j in 1:p){
        if(glmfamily=="poisson"){
            cdf.data.m[i,j] <- (ppois(y.mat[i,j],std.glm.means[i,j])+
                                ppois(y.mat[i,j]-1,std.glm.means[i,j]))/2
            ## Trap outliers which give NA:
            if (ppois(y.mat[i,j]-1,fits.m[i,j]) > 0.999999)
                rqresid.m[i,j] <- 1
            if (ppois(y.mat[i,j]-1,fits.m[i,j]) <= 0.999999)
                rqresid.m[i,j] <- runif(1,min = max(0,ppois(y.mat[i,j]-1,fits.m[i,j])),
                                        max = ppois(y.mat[i,j],fits.m[i,j]))
        }
        if(glmfamily=="bernoulli")
            cdf.data.m[i,j] <- (pbinom(y.mat[i,j],size=1,std.glm.means[i,j])+
                                pbinom(y.mat[i,j]-1,size=1,std.glm.means[i,j]))/2
        if(glmfamily=="binomial")
            cdf.data.m[i,j] <- (pbinom(y.mat[i,j],size=ntrials,std.glm.means[i,j])+
                                pbinom(y.mat[i,j]-1,size=ntrials,std.glm.means[i,j]))/2
        if(glmfamily=="gaussian"){
            var.s <- std.glm[[10]]/std.glm[[16]]
            cdf.data.m[i,j] <- (pnorm(y.mat[i,j],std.glm.means[i,j],sd=sqrt(var.s))+
                                pnorm(y.mat[i,j]-1,std.glm.means[i,j],sd=sqrt(var.s)))/2
        }
    }
    
    
    ## Output results:
    y.vect <- std.glm$y
    if (glmfamily=="binomial") y.vect <- y.vect*ntrials
    y.mat <- matrix(y.vect,n,p)
    rownames(y.mat) <- as.character(1:n)
    colnames(y.mat) <- as.character(1:p)
    rnames <- rownames(y.mat)
    cnames <- colnames(y.mat)
    y.in <- y.mat
    LL <- -std.glm$deviance/2
    npar <- glm.npar
    res.dev <- std.glm$deviance
    LLc <- LL
    entR <- 0
    entC <- 0
    Nobs <- n*p
    aic  <- res.dev+2*npar
    aicc <- NA
    if (Nobs > npar+2)
        aicc <- aic+(2*(npar+1)*(npar+2))/(Nobs-npar-2)
    bic <- -2*LL+npar*log.fn(Nobs)
    icl <- -2*LLc+npar*log.fn(Nobs)
    RG <- NA
    CG <- NA
    
    out1 <- round(c(n,p,LL,res.dev,npar,aic,aicc,bic,icl,LLc,
                    RG,CG,entR,entC),3)
    names(out1) <- c("n","p","Max.ll","Res.Dev.","npar","AIC","AICc",
                     "BIC","ICL","LLc","RG","CG","EntR","EntC")
    if(glmfamily=="binomial")
        out1 <- c(out1,"ntrials"=ntrials)
    
    this.glm <- std.glm
    pi.v <- 1
    kappa.v <- 1
    nuab.mat <- matrix(predict(std.glm),n,p)
    nu.s <- mean(nuab.mat)
    alpha.v <- apply(nuab.mat,1,mean)-nu.s
    names(alpha.v) <- rnames
    beta.v <- apply(nuab.mat,2,mean)-nu.s
    names(beta.v) <- cnames
    gamma.m <- matrix(0,1,1)
    ppr.m <- matrix(1,n,1)
    rownames(ppr.m) <- rnames
    ppc.m <- matrix(1,p,1)
    rownames(ppc.m) <- cnames
    matrix.fits <- matrix(fitted(this.glm),n,p)
    dimnames(matrix.fits) <- list(rnames,cnames)
    lin.a <- array(matrix.fits,c(n,p,1,1))
    dimnames(lin.a) <- list(rnames,cnames,c("1"),c("1"))
    ngammas <- 0
    
    model.out <- list("info"=out1,
                      "formula"=glmformula,
                      "family"=glmfamily,
                      "data"=data,
                      "std.glm" = std.glm,
                      "this.model.glm" = std.glm,
                      "pi"=pi.v,
                      "kappa"=kappa.v,
                      "nu"=nu.s,
                      "alpha"=alpha.v,
                      "beta"=beta.v,
                      "gamma"=gamma.m,
                      "resid.var"=var.s,
                      "cdf.data"=cdf.data.m,
                      "cdf.group"=NULL,
                      "cdf.factorgroup1"=NULL,
                      "cdf.factorgroup2"=NULL,
                      "response.fits"=fits.m,
                      "rand.quant.resids"=rqresid.m,
                      "ppr"=ppr.m,
                      "ppc"=ppc.m,
                      "Factor 1 groups"=NULL,
                      "Factor 2 groups"=NULL)
    
    print("std.glm stored")
    
    if(is.null(clustfact))
        print("This is a simple glm, no clustering")
    
    
    if (length(clustfact)>0){
        ## Set up the clustering:
        
        print("Setting up clustering of factors")
        print(clustfact)
        ## Error messages if not correctly specified
        if (length(clustfact) != length(numbersclusters))
            stop("Please specify the number of clusters for each clustered factor")
        if (any(floor(numbersclusters)==numbersclusters)<1)
            stop("Please use integers for numbers of clusters")
        for (cf in 1:length(clustfact)){
            if (!(clustfact[cf] %in% names(data)))
                stop(paste("object",clustfact[cf],"not found"))
            if (!is.factor(data[,names(data)==clustfact[cf]]))
                stop(paste(clustfact[cf],"is not a factor"))
        }
        
        if(max(numbersclusters)>1){
            ## Assume data from a matrix.
            ## Find which columns of the data frame have factors used in the formula:
            
            mefacts <- rep(NA,length(clustfact))
            nlevels <- rep(NA,length(clustfact))
            for (cf in 1:length(clustfact)){
                mefacts[cf] <- (1:ncol(data))[clustfact[cf] == names(data)]
                nlevels[cf] <- length(levels(data[,mefacts[cf]]))
            }    
            
            ## Find the type of standardisation requested:
            maineffects <- attr(terms(glmformula),"term.labels")[attr(terms(glmformula),
                                                                      "order")==1]
            std <- "N"
            if(length(maineffects)==2){
                if((maineffects[1] %in% clustfact) & (maineffects[2] %in% clustfact))
                    std <- "D"
                if((maineffects[1] %in% clustfact) & (!(maineffects[2] %in% clustfact)))
                    std <- "A"
                if((!(maineffects[1] %in% clustfact)) & (maineffects[2] %in% clustfact))
                    std <- "B"
            }
            if(length(maineffects)==1){
                if(maineffects[1]=="ROW")
                    std <- "A"
                if(maineffects[1]=="COL")
                    std <- "B"
            }
            print(paste("Type of standardization is",std))
            
            ## If clustering of RG=n or CG=p has been requested, ensure only
            ## single-mode clustering is done. Set flags for non-trivial clusterings.
            clustrows <- FALSE
            clustcols <- FALSE
            clustboth <- FALSE
            for (cf in 1:length(clustfact)){
                if (clustfact[cf] == "ROW"){
                    nrowclust <- numbersclusters[cf]
                    if ((nrowclust>1)&(nrowclust<n)) clustrows <- TRUE
                } 
                if (clustfact[cf] == "COL"){
                    ncolclust <- numbersclusters[cf]
                    if ((ncolclust>1)&(ncolclust<p)) clustcols <- TRUE
                }
            }
            if ((clustrows==1)&(clustcols==1))
                clustboth <- TRUE
            print(paste("Clustrows =",clustrows))
            print(paste("Clustcols =",clustcols))
            print(paste("Clustboth =",clustboth))
            
            ## Select the appropriate M.step.fn:
            if ((clustrows == T)&(clustcols == T)){
                M.step.fn <- M.step.bi.fn
                print("Using M.step.bi.fn")
            } 
            if ((clustrows == T)&(clustcols == F)){
                M.step.fn <- M.step.rows.fn 
                print("Using M.step.rows.fn")
            } 
            if ((clustrows == F)&(clustcols == T)){
                M.step.fn <- M.step.cols.fn 
                print("Using M.step.cols.fn")
            } 
            
            ## For each clustering request, lengthen the data frame
            ## and include temporary values of levels of the new variable.
            currentdata <- data
            for (cf in 1:length(clustfact)){
                nlevels <- numbersclusters[cf]
                longer.df <- currentdata
                if ((nlevels>1)&(nlevels<c(n,p)[cf]))
                    for (repl in 2:nlevels)
                        longer.df <- rbind(longer.df,currentdata)
                longer.df$x <- gl(nlevels,nrow(currentdata),nrow(longer.df))
                names(longer.df)[ncol(longer.df)] <- clustfactnames[cf]
                currentdata <- longer.df
            }
            
            long.df <- longer.df
            rownames(long.df) <- as.character(1:nrow(long.df))
            
            
            ## Find numbers of levels of all factors in long.df:
            
            nlevels <- rep(NA,ncol(long.df))
            for (j in 1:ncol(long.df))
                if (is.factor(long.df[,j]))
                    nlevels[j] <- length(levels(long.df[,j]))
            names(nlevels) <- names(long.df)
            
            print("Numbers of levels of factors")
            print(nlevels)
            
            RG <- numbersclusters[1]
            CG <- 1
            if (length(numbersclusters)==2)
                CG <- numbersclusters[2]
            y.a <- array(y.mat,c(n,p,RG,CG))    
            
            
            
            ## Starts for other cases:
            
            if ((RG>1)|(CG>1)){
                
                ## Given starts:
                
                ## Use given starts, if supplied:
                if((!is.null(ppr.start))|(!is.null(ppc.start))){
                    ## Row clustering:
                    if(((RG>1)&(RG<n))&((CG==1)|(CG==p))&(!is.null(ppr.start))){
                        ppr.m <- ppr.start
                        if (CG==1) ppc.m <- matrix(1,p,1)
                        if (CG==p) ppc.m <- diag(p)
                    }
                    ## Column clustering:
                    if(((RG==1)|(RG==n))&((CG>1)&(CG<p))&(!is.null(ppc.start))){
                        ppc.m <- ppc.start
                        if (RG==1) ppr.m <- matrix(1,n,1)
                        if (RG==n) ppr.m <- diag(n)
                    }
                    ## Biclustering:
                    if(((RG>1)&(RG<n))&((CG>1)&(CG<p))&
                       (!is.null(ppr.start))&(!is.null(ppc.start))){
                        ppr.m <- ppr.start
                        ppc.m <- ppc.start
                    }
                    
                    print("Input ppr")
                    print(ppr.m)
                    print("Input ppc")
                    print(ppc.m)
                    
                    if(((RG>1)&(RG<n))&((CG==1)|(CG==p))&(is.null(ppr.start)))
                        print("ppr.start not ppc.start needs to be provided for this model!")
                    if(((RG==1)|(RG==n))&((CG>1)&(CG<p))&(is.null(ppc.start)))
                        print("ppc.start not ppr.start needs to be provided for this model!")
                    if(((RG>1)&(RG<n))&((CG>1)&(CG<p))&
                       ((is.null(ppr.start))|(is.null(ppc.start))))
                        print("Both ppr.start and ppc.start needs to be provided for 
                              this model!")
                    
                    ## Preliminary M step from the starting ppr, ppc:
                    print("Preliminary M step")
                    prelim.M.step <- M.step.fn(glmfamily,
                                               n,p,
                                               RG,CG,
                                               ppr.m,ppc.m,
                                               clustformula,
                                               long.df,std.glm)
                    lin.a <- prelim.M.step[[1]]
                    pi.v <- prelim.M.step[[2]]
                    kappa.v <- prelim.M.step[[3]]
                    LL <- prelim.M.step[[4]]
                    var.s <- prelim.M.step[[6]]
                }   ## End of given ppr, ppc starts
                
                
                
                ## If no given starts, use some options:
                if((is.null(ppr.start))&(is.null(ppc.start))){
                    options(warn=-1)
                    
                    ## Need adjusted quantile matrix for use in informed starts.
                    ## Define adjusted quantile vector:
                    if (glmfamily=="poisson")
                        aq.vect <- (ppois(std.glm$y,fitted(std.glm)) +
                                    ppois(std.glm$y-1,fitted(std.glm)))/2
                    if (glmfamily=="bernoulli")
                        aq.vect <- (pbinom(std.glm$y,1,fitted(std.glm)) +
                                    pbinom(std.glm$y-1,1,fitted(std.glm)))/2
                    if (glmfamily=="binomial")   #### May need *ntrials
                        aq.vect <- (pbinom(std.glm$y*ntrials,ntrials,fitted(std.glm)) +
                                    pbinom(std.glm$y-1,ntrials,fitted(std.glm)))/2
                    if (glmfamily=="gaussian")
                        aq.vect <- (pnorm(std.glm$y,fitted(std.glm),sd=sqrt(var.s)) +
                                    pnorm(std.glm$y-1,fitted(std.glm),sd=sqrt(var.s)))/2
                    ## Define adjusted quantile matrix:                
                    aq.mat <- matrix(aq.vect,n,p)
                    
                    
                    ## Option 1 - Kmeans start:
                    ## ------------------------
                    
                    print("Beginning k-means starts")
                    
                    ## Define initial ppr and ppc matrices:
                    
                    ## Set fixed ppr, ppc or flags for degenerate cases:
                    if (RG==1) ppr.m <- matrix(1,n,1)
                    if (RG==n) ppr.m <- diag(n)
                    if (CG==1) ppc.m <- matrix(1,p,1)
                    if (CG==p) ppc.m <- diag(p)
                    
                    ## Row clustering:
                    ## Full CG=p case:
                    if ((RG>1)&(RG<n)&(CG==p)){
                        gps <- kmeans(aq.mat,RG,nstart=kstarts)[[1]]
                        ppr.m <- matrix(0,n,RG)
                        for (i in 1:n) ppr.m[i,gps[i]] <- 1
                    }
                    ## CG=1 case:
                    if ((RG>1)&(RG<n)&(CG==1)){
                        gps <- kmeans(apply(aq.mat,1,mean),RG,nstart=kstarts)[[1]]
                        ppr.m <- matrix(0,n,RG)
                        for (i in 1:n) ppr.m[i,gps[i]] <- 1
                    }
                    ## Column clustering:
                    ## Full RG=n case:
                    if ((CG>1)&(CG<p)&(RG==n)){
                        gps <- kmeans(t(aq.mat),CG,nstart=kstarts)[[1]]
                        ppc.m <- matrix(0,p,CG)
                        for (j in 1:p) ppc.m[j,gps[j]] <- 1
                    }
                    ## RG=1 case:
                    if ((CG>1)&(CG<p)&(RG==1)){
                        gps <- kmeans(apply(aq.mat,2,mean),CG,nstart=kstarts)[[1]]
                        ppc.m <- matrix(0,p,CG)
                        for (j in 1:p) ppc.m[j,gps[j]] <- 1
                    }
                    ## Biclustering:
                    if (((RG>1)&(RG<n))&((CG>1)&(CG<p))){
                        startlist <- dkm.fn(aq.mat,RG,CG,kstarts)
                        ppr.m <- startlist[[1]]
                        ppc.m <- startlist[[2]]
                    }

                    
                    ## Preliminary M step from the starting ppr, ppc:
                    Kmeans.M.step <- M.step.fn(glmfamily,
                                               n,p,
                                               RG,CG,
                                               ppr.m,ppc.m,
                                               clustformula,
                                               long.df,std.glm)
                    lin.a <- Kmeans.M.step[[1]]
                    pi.v <- Kmeans.M.step[[2]]
                    kappa.v <- Kmeans.M.step[[3]]
                    var.s <- Kmeans.M.step[[6]]
                    
                    ## Run some cycles (default=3) of the EM algorithm to find a 
                    ## log likelihood
                    for(Mstartcycles in 1:startEMcycles){
                        Kmeans.E.step <- E.step.fn(std,glmfamily,lin.a,y.a,
                                                   n,p,RG,CG,pi.v,
                                                   kappa.v,ppr.m,ppc.m,ntrials,var.s)
                        
                        ppr.m <- Kmeans.E.step[[1]]
                        ppc.m <- Kmeans.E.step[[2]]
                        Kmeans.M.step <- M.step.fn(glmfamily,
                                                   n,p,
                                                   RG,CG,
                                                   ppr.m,ppc.m,
                                                   clustformula,
                                                   long.df,std.glm)
                    }
                    
                    best.lin.a <- lin.a
                    best.pi.v <- pi.v
                    best.kappa.v <- kappa.v
                    best.ppr.m <- ppr.m
                    best.ppc.m <- ppc.m
                    best.LL <- Kmeans.M.step[[4]]
                    best.var.s <- var.s
                    
                    print("Finished k-means starts, best chosen")
                    
                    
                    ## Option 2 - Hclust start:
                    ## ------------------------
                    
                    if(Hclust==TRUE){
                        print("Beginning Hclust starts")
                        
                        ## Rows: do this if 1<RG<n
                        this.dist <- dist(aq.mat,method="manhattan")
                        this.hclust <- hclust(this.dist,method="complete") 
                        RGps <- cutree(this.hclust,k=RG)
                        ppr.m <- matrix(0,n,RG)
                        for (i in 1:n) ppr.m[i,RGps[i]] <- 1
                        
                        ## Cols: do this if 1<CG<p
                        this.dist <- dist(t(aq.mat),method="manhattan")
                        this.hclust <- hclust(this.dist,method="complete")
                        CGps <- cutree(this.hclust,k=CG)
                        ppc.m <- matrix(0,p,CG)
                        for (j in 1:p) ppc.m[j,CGps[j]] <- 1
                        
                        ## Preliminary M step from the starting ppr, ppc:
                        Hclust.M.step <-M.step.fn(glmfamily,
                                                  n,p,
                                                  RG,CG,
                                                  ppr.m,ppc.m,
                                                  clustformula,
                                                  long.df,std.glm)
                        lin.a <- Hclust.M.step[[1]]
                        pi.v <- Hclust.M.step[[2]]
                        kappa.v <- Hclust.M.step[[3]]
                        var.s <- Hclust.M.step[[6]]
                        
                        ## Run some cycles (default=3) of the EM algorithm to find a 
                        ## log likelihood
                        for(Mstartcycles in 1:startEMcycles){
                            Hclust.E.step <-E.step.fn(std,glmfamily,lin.a,y.a,
                                                      n,p,RG,CG,pi.v,kappa.v,
                                                      ppr.m,ppc.m,ntrials,var.s)
                            ppr.m <- Hclust.E.step[[1]]
                            ppc.m <- Hclust.E.step[[2]]
                            Hclust.M.step <- M.step.fn(glmfamily,
                                                       n,p,
                                                       RG,CG,
                                                       ppr.m,ppc.m,
                                                       clustformula,
                                                       long.df,std.glm)
                            lin.a <- Hclust.M.step[[1]]
                            pi.v <- Hclust.M.step[[2]]
                            kappa.v <- Hclust.M.step[[3]]
                            var.s <- Hclust.M.step[[6]]
                        }
                        
                        this.lin.a <- lin.a
                        this.pi.v <- pi.v
                        this.kappa.v <- kappa.v
                        this.ppr.m <- ppr.m
                        this.ppc.m <- ppc.m
                        this.LL <- Hclust.M.step[[4]]
                        this.var.s <- var.s
                        
                        if(this.LL>best.LL){
                            best.lin.a <- this.lin.a
                            best.pi.v <- this.pi.v
                            best.kappa.v <- this.kappa.v
                            best.ppr.m <- this.ppr.m
                            best.ppc.m <- this.ppc.m
                            best.LL <- this.LL
                            best.var.s <- this.var.s
                            print("Better start found")
                        }
                        
                        print("Finished Hclust starts")
                    }
                    
                    
                    ## Option 3 - Diana method:
                    ## ------------------------
                    
                    if(Diana == TRUE){
                        print("Beginning Diana method starts")
                        
                        ## Rows:
                        this.diana <- diana(aq.mat,metric="manhattan",
                                            keep.diss=FALSE,keep.data=FALSE)
                        RGps <- cutree(as.hclust(this.diana),k=RG)
                        ppr.m <- matrix(0,n,RG)
                        for (i in 1:n) ppr.m[i,RGps[i]] <- 1
                        
                        ## Cols:
                        this.diana <- diana(t(aq.mat),metric="manhattan",
                                            keep.diss=FALSE,keep.data=FALSE)
                        CGps <- cutree(as.hclust(this.diana),k=CG)
                        ppc.m <- matrix(0,p,CG)
                        for (j in 1:p) ppc.m[j,CGps[j]] <- 1
                        
                        ## Preliminary M step from the starting ppr, ppc:
                        diana.M.step <- M.step.fn(glmfamily,
                                                  n,p,
                                                  RG,CG,
                                                  ppr.m,ppc.m,
                                                  clustformula,
                                                  long.df,std.glm)
                        lin.a <- diana.M.step[[1]]
                        pi.v <- diana.M.step[[2]]
                        kappa.v <- diana.M.step[[3]]
                        var.s <- diana.M.step[[6]]
                        
                        ## Run some cycles (default=3) of the EM algorithm to find a 
                        ## log likelihood
                        for(Mstartcycles in 1:startEMcycles){
                            diana.E.step <- E.step.fn(std,glmfamily,lin.a,y.a,
                                                      n,p,RG,CG,pi.v,kappa.v,
                                                      ppr.m,ppc.m,ntrials,var.s)
                            ppr.m <- diana.E.step[[1]]
                            ppc.m <- diana.E.step[[2]]
                            diana.M.step <- M.step.fn(glmfamily,
                                                      n,p,
                                                      RG,CG,
                                                      ppr.m,ppc.m,
                                                      clustformula,
                                                      long.df,std.glm)
                            lin.a <- diana.M.step[[1]]
                            pi.v <- diana.M.step[[2]]
                            kappa.v <- diana.M.step[[3]]
                            var.s <- diana.M.step[[6]]
                        }
                        
                        this.lin.a <- lin.a
                        this.pi.v <- pi.v
                        this.kappa.v <- kappa.v
                        this.ppr.m <- ppr.m
                        this.ppc.m <- ppc.m
                        this.LL <- diana.M.step[[4]]
                        this.var.s <- var.s
                        
                        if(this.LL>best.LL){
                            best.lin.a <- this.lin.a
                            best.pi.v <- this.pi.v
                            best.kappa.v <- this.kappa.v
                            best.ppr.m <- this.ppr.m
                            best.ppc.m <- this.ppc.m
                            best.LL <- this.LL
                            best.var.s <- this.var.s
                            print("Better start found")
                        }
                        
                        print("Finished Diana method starts")
                    }
                    
                    
                    ## Option 4 - Random start for parameters:
                    ## ---------------------------------------
                    
                    if(randstarts>0){
                        print(paste("Beginning",randstarts,"random starts for parameters"))
                        print(paste("Each random start is run for",startEMcycles,
                                    "EM cycles", sep=" "))
                        
                        
                        for(startiter in 1:randstarts){
                            
                            print(paste("Random start",startiter))
                            
                            ## Define link function:
                            if (glmfamily=="poisson")
                                link.fn <- log.fn
                            if (glmfamily=="bernoulli")
                                link.fn <- logit.fn
                            if (glmfamily=="binomial")
                                link.fn <- logit.fn
                            if (glmfamily=="gaussian")
                                link.fn <- id.fn
                            
                            gy.in  <- link.fn(y.in)
                            
                            res.mat <- gy.in - fitted(std.glm)
                            resids <- c(res.mat)
                            
                            gamma.m <- matrix(sample(resids,RG*CG),RG,CG)
                            
                            
                            if ((RG>1)&(RG<n)){
                                pi.v <- runif(RG,0.4,0.6)
                                pi.v <- pi.v/sum(pi.v)
                                ppr.m <- matrix(pi.v,n,RG,byrow=T)
                            }
                            if ((CG>1)&(CG<p)){
                                kappa.v <- runif(CG,0.4,0.6)
                                kappa.v <- kappa.v/sum(kappa.v)
                                ppc.m <- matrix(kappa.v,p,CG,byrow=T)
                            }
                            
                            ## Set fixed pi, kappa, ppr, ppc for degenerate cases:
                            if (RG==1){
                                ppr.m <- matrix(1,n,1)
                                pi.v <- 1
                            }
                            if (RG==n){
                                ppr.m <- diag(n)
                                pi.v <- rep(1/n,n)
                            }
                            if (CG==1){
                                ppc.m <- matrix(1,p,1)
                                kappa.v <- 1
                            }
                            if (CG==p){
                                ppc.m <- diag(p)
                                kappa.v <- rep(1/p,p)
                            }
                            
                            ## Need linear array:
                            nuab.a <- matrix(fitted(std.glm),n,p)
                            lin.a  <- outer(nuab.a,gamma.m,FUN="+")
                            
                            ## Run some cycles (default=3) of the EM algorithm to find a
                            ## log likelihood
                            for(Mstartcycles in 1:startEMcycles){
                                rand.starts.E.step <-
                                    E.step.fn(std,glmfamily,lin.a,y.a,n,p,RG,CG,pi.v,
                                              kappa.v,ppr.m,ppc.m,ntrials,var.s)
                                ppr.m <- rand.starts.E.step[[1]]
                                ppc.m <- rand.starts.E.step[[2]]
                                rand.starts.M.step <- M.step.fn(glmfamily,
                                                                n,p,
                                                                RG,CG,
                                                                ppr.m,ppc.m,
                                                                clustformula,
                                                                long.df,std.glm)
                                lin.a <- rand.starts.M.step[[1]]
                                pi.v <- rand.starts.M.step[[2]]
                                kappa.v <- rand.starts.M.step[[3]]
                                var.s <- rand.starts.M.step[[6]]
                            }
                            
                            this.lin.a <- lin.a
                            this.pi.v <- pi.v
                            this.kappa.v <- kappa.v
                            this.ppr.m <- ppr.m
                            this.ppc.m <- ppc.m
                            this.LL <- rand.starts.M.step[[4]]
                            this.var.s <- var.s
                            
                            if((this.LL>best.LL)&(min(apply(ppr.m,2,sum))>0)&
                               (min(apply(ppc.m,2,sum))>0)){
                                best.lin.a <- this.lin.a
                                best.pi.v <- this.pi.v
                                best.kappa.v <- this.kappa.v
                                best.ppr.m <- this.ppr.m
                                best.ppc.m <- this.ppc.m
                                best.LL <- this.LL
                                best.var.s <- this.var.s
                                print("Better start found")
                            }
                            
                        }
                        
                        print("Finished random starts for parameters")
                    }
                    
                    
                    ## Option 5 - Split RGs from a nominated model with fewer RGs:
                    ## -----------------------------------------------------------
                    
                    if((1<RG)&(RG<n)&(is.list(RGsplit))){
                        
                        print("Beginning splitting RGs from a nominated model with fewer RGs")
                        ppr.in <- cbind(RGsplit$pp.ROW,rep(0,n))
                        
                        if(ncol(ppr.in)!=RG)
                            stop("For splitting a row group, need an input model with one 
                                 less RG than the current model.")
                        if(min(apply(ppr.in[,-RG],2,sum))==0)
                            stop("For splitting a row group, need an input model with RG-1 
                                 distinct groups")
                        
                        for(ii in 1:n){
                            print(paste("Splitting off row",ii))
                            ppr.m <- ppr.in
                            ppr.m[ii,] <- c(rep(0,RG-1),1)
                            
                            ## Run some cycles (default=3) of the EM algorithm to find
                            ## a log likelihood
                            if(min(apply(ppr.m,2,sum))>0){
                                for(Mstartcycles in 1:startEMcycles){
                                    rand.starts.E.step <-
                                        E.step.fn(std,glmfamily,lin.a,y.a,n,p,RG,CG,
                                                  pi.v,kappa.v,ppr.m,ppc.m,ntrials,var.s)
                                    ppr.m <- rand.starts.E.step[[1]]
                                    ppc.m <- rand.starts.E.step[[2]]
                                    rand.starts.M.step <- M.step.fn(glmfamily,
                                                                    n,p,
                                                                    RG,CG,
                                                                    ppr.m,ppc.m,
                                                                    clustformula,
                                                                    long.df,std.glm)
                                    lin.a <- rand.starts.M.step[[1]]
                                    pi.v <- rand.starts.M.step[[2]]
                                    kappa.v <- rand.starts.M.step[[3]]
                                    var.s <- rand.starts.M.step[[6]]
                                }
                                
                                this.lin.a <- lin.a
                                this.pi.v <- pi.v
                                this.kappa.v <- kappa.v
                                this.ppr.m <- ppr.m
                                this.ppc.m <- ppc.m
                                this.LL <- rand.starts.M.step[[4]]
                                this.var.s <- var.s
                                
                                if((this.LL>best.LL)&(min(apply(ppr.m,2,sum))>0)&
                                   (min(apply(ppc.m,2,sum))>0)){
                                    best.lin.a <- this.lin.a
                                    best.pi.v <- this.pi.v
                                    best.kappa.v <- this.kappa.v
                                    best.ppr.m <- this.ppr.m
                                    best.ppc.m <- this.ppc.m
                                    best.LL <- this.LL
                                    best.var.s <- this.var.s
                                    print("Better start found")
                                }
                            }
                        }
                        print("Finished splitting RGs from a nominated model with fewer RGs")
                    }  ## End of Option 5.
                    
                    
                    ## Option 6 - Split CGs from a nominated model with fewer CGs:
                    ## -----------------------------------------------------------
                    
                    if((1<CG)&(CG<p)&(is.list(CGsplit))){
                        print("Beginning splitting CGs from a nominated model with fewer CGs")
                        ppc.in <- cbind(CGsplit$pp.COL,rep(0,p))
                        
                        if(ncol(ppc.in)!=CG)
                            stop("For splitting a column group, need an input model with one 
                                 less CG than the current model.")
                        if(min(apply(ppc.in[,-CG],2,sum))==0)
                            stop("For splitting a column group, need an input model with CG-1 
                                 distinct groups")
                        
                        for(jj in 1:p){
                            print(paste("Splitting off column",jj))
                            ppc.m <- ppc.in
                            ppc.m[jj,] <- c(rep(0,CG-1),1)
                            
                            ## Run some cycles (default=3) of the EM algorithm to find
                            ## a log likelihood
                            if(min(apply(ppc.m,2,sum))>0){
                                for(Mstartcycles in 1:startEMcycles){
                                    rand.starts.E.step <- E.step.fn(std,glmfamily,lin.a,y.a,
                                                                    n,p,RG,CG,pi.v,kappa.v,
                                                                    ppr.m,ppc.m,ntrials,var.s)
                                    ppr.m <- rand.starts.E.step[[1]]
                                    ppc.m <- rand.starts.E.step[[2]]
                                    rand.starts.M.step <- M.step.fn(glmfamily,
                                                                    n,p,
                                                                    RG,CG,
                                                                    ppr.m,ppc.m,
                                                                    clustformula,
                                                                    long.df,std.glm)
                                    lin.a <- rand.starts.M.step[[1]]
                                    pi.v <- rand.starts.M.step[[2]]
                                    kappa.v <- rand.starts.M.step[[3]]
                                    var.s <- rand.starts.M.step[[6]]
                                }
                                
                                this.lin.a <- lin.a
                                this.pi.v <- pi.v
                                this.kappa.v <- kappa.v
                                this.ppr.m <- ppr.m
                                this.ppc.m <- ppc.m
                                this.LL <- rand.starts.M.step[[4]]
                                this.var.s <- var.s
                                
                                if((this.LL>best.LL)&(min(apply(ppr.m,2,sum))>0)&
                                   (min(apply(ppc.m,2,sum))>0)){
                                    best.lin.a <- this.lin.a
                                    best.pi.v <- this.pi.v
                                    best.kappa.v <- this.kappa.v
                                    best.ppr.m <- this.ppr.m
                                    best.ppc.m <- this.ppc.m
                                    best.LL <- this.LL
                                    best.var.s <- this.var.s
                                    print("Better start found")
                                }
                            }
                        }
                        print("Finished splitting CGs from a nominated model with fewer CGs")
                    }  ## End of Option 6.
                    
                    ## Select the best starting point
                    lin.a <- best.lin.a
                    ppr.m <- best.ppr.m
                    ppc.m <- best.ppc.m
                    pi.v <- best.pi.v
                    kappa.v <- best.kappa.v
                    LL <- best.LL
                    var.s <- best.var.s
                }  ## End of automated starts when ppr, ppc starts not given.
                
                
                ## Run the EM cycle on the best start until the parameters have stabilised:
                ## ------------------------------------------------------------------------
                
                options(warn=0)
                print("Starting EM algorithm")
                
                EMoutvect <- c(LL,c(lin.a),pi.v,kappa.v)
                EMinvect  <- rep(1,length(EMoutvect))
                EMiter <- 1
                
                while(((EMiter==1)|(any(abs(EMinvect-EMoutvect)>EMstoppingpar)))&
                      (EMiter<(EMcycles+1))){
                    print(paste("EM iteration",EMiter))
                    full.cycle.E.step <- E.step.fn(std,glmfamily,lin.a,y.a,
                                                   n,p,RG,CG,pi.v,
                                                   kappa.v,ppr.m,ppc.m,ntrials,var.s)
                    ppr.m <- full.cycle.E.step[[1]]
                    ppc.m <- full.cycle.E.step[[2]]
                    full.cycle.M.step <- M.step.fn(glmfamily,
                                                   n,p,
                                                   RG,CG,
                                                   ppr.m,ppc.m,
                                                   clustformula,
                                                   long.df,std.glm)
                    lin.a <- full.cycle.M.step[[1]]
                    pi.v <- full.cycle.M.step[[2]]
                    kappa.v <- full.cycle.M.step[[3]]
                    LL <- full.cycle.M.step[[4]]
                    var.s <- full.cycle.M.step[[6]]
                    
                    ## Update iteration number, vector of pars:
                    EMiter <- EMiter + 1
                    EMinvect <- EMoutvect
                    EMoutvect <- c(LL,c(lin.a),pi.v,kappa.v)
                }
                
                ## End of EM cycle
                if (EMiter<EMcycles)
                    print(paste("EM algorithm converged after",EMiter-1,"iterations",sep=" "))
                if (EMiter>(EMcycles-1))
                    print(paste("EM algorithm stopped after",EMiter-1,"iterations",sep=" "))
                ## Save results for output:
                lc1 <- - full.cycle.M.step$this.glm$deviance/2
                LLc <- lc1 + (RG>1)*(RG<n)*n*sum(xlogx.fn(pi.v)) +
                    (CG>1)*(CG<p)*p*sum(xlogx.fn(kappa.v))
                entR <- -sum(xlogx.fn(ppr.m))
                entC <- -sum(xlogx.fn(ppc.m))
                ## Following npar is correct if have a cov. in interaction:
                glm.npar <- full.cycle.M.step$npar
                npar <- glm.npar + (RG<n)*(RG-1) + (CG<p)*(CG-1)
                ## Above npar may be wrong, from using R's glm.
                ## glm.npar can't be trusted, calculate separately below.
                ## R glm with non-integer weights estimates too many pars,
                ## doesn't recognise redundancies. Computer rounding.
                ## Will need to deal with npar for covariates.
                this.glm <- full.cycle.M.step$this.glm
            }
            
            ## Warning messages
            ## Failure to distinguish enough groups:
            if (min(pi.v)==0) print(paste("Cannot distinguish",RG,
                       "row groups. Try re-running the model."))
            if (min(kappa.v)==0) print(paste("Cannot distinguish",CG,
                       "column groups. Try re-running the model."))
            
            
            ## Simple glms with redundant parameters:
            if((std=="A")&(RG>1)&(CG==1)){
                print("This is the same as the A11 model.")
                npar <- n
            }
            if((std=="N")&(RG==n)&(CG==1)){
                print("This is the same as the A11 model.")
                npar <- n
            }
            if((std=="B")&(RG==1)&(CG>1)){
                print("This is the same as the B11 model.")
                npar <- p
            }
            if((std=="N")&(RG==1)&(CG==p)){
                print("This is the same as the B11 model.")
                npar <- p
            }
            if((std=="D")&(((RG>1)&(CG==1))|((RG==1)&(CG>1)))){
                print("This is the same as the D11 model.")
                npar <- n + p - 1
            }
            if((std=="A")&(RG==1)&(CG==p)){
                print("This is the same as the D11 model.")
                npar <- n + p - 1
            }
            if((std=="B")&(RG==n)&(CG==1)){
                print("This is the same as the D11 model.")
                npar <- n + p - 1
            }
            ## Calculate correct npar for non-covariate models:
            if ((cov.interaction == F)&(0==1)){
                ## Clustered models with redundant parameters:
                ## NRp = BRp
                for(i in 2:(n-1)){
                    if((std=="N")&(RG==i)&(CG==p)){
                        print(paste("This is the same as the B",i,"p model",sep=""))
                        npar <- RG*p + (RG - 1)
                    }
                    if((std=="B")&(RG==i)&(CG==p)){
                        print(paste("This is the same as the N",i,"p model",sep=""))
                        npar <- RG*p + (RG - 1)
                    }
                }
                ## ARp = DRp
                for(i in 2:(n-1)){
                    if((std=="A")&(RG==i)&(CG==p)){
                        print(paste("This is the same as the D",i,"p model",sep=""))
                        npar <- n + RG*(p-1) + (RG - 1)
                    }
                    if((std=="D")&(RG==i)&(CG==p)){
                        print(paste("This is the same as the A",i,"p model",sep=""))
                        npar <- n + RG*(p-1) + (RG - 1)
                    }
                }
                ## NnC = AnC
                for(j in 2:(p-1)){
                    if((std=="N")&(RG==n)&(CG==j)){
                        print(paste("This is the same as the An",j," model",sep=""))
                        npar <- n*CG + (CG - 1)
                    }
                    if((std=="A")&(RG==n)&(CG==j)){
                        print(paste("This is the same as the Nn",j," model",sep=""))
                        npar <- n*CG + (CG - 1)
                    }
                }
                ## BnC = DnC
                for(j in 2:(p-1)){
                    if((std=="B")&(RG==n)&(CG==j)){
                        print(paste("This is the same as the Dn",j," model",sep=""))
                        npar <- p + (n-1)*CG + (CG - 1)
                    }
                    if((std=="D")&(RG==n)&(CG==j)){
                        print(paste("This is the same as the Bn",j," model",sep=""))
                        npar <- p + (n-1)*CG + (CG - 1)
                    }
                }
                ## Find npar for other cases, individual models, no redundancy:
                ## NR1:
                for(i in 2:(n-1))
                    if((std=="N")&(RG==i)&(CG==1))
                        npar <- RG + (RG - 1)
                ## N1C:
                for(j in 2:(p-1))
                    if((std=="N")&(RG==1)&(CG==j))
                        npar <- CG + (CG - 1)
                ## NRC:
                for(i in 2:(n-1)) for(j in 2:(p-1))
                    if((std=="N")&(RG==i)&(CG==j))
                        npar <- RG*CG + (RG - 1) + (CG - 1)
                ## A1C:
                for(j in 2:(p-1))
                    if((std=="A")&(RG==1)&(CG==j))
                        npar <- n + (CG - 1) + (CG - 1)
                ## ARC:
                for(i in 2:(n-1)) for(j in 2:(p-1))
                    if((std=="A")&(RG==i)&(CG==j))
                        npar <- n + RG*(CG - 1) + (RG - 1) + (CG - 1)
                ## BR1:
                for(i in 2:(n-1))
                    if((std=="B")&(RG==i)&(CG==1))
                        npar <- p + (RG - 1) + (RG - 1)
                ## BRC:
                for(i in 2:(n-1)) for(j in 2:(p-1))
                    if((std=="B")&(RG==i)&(CG==j))
                        npar <- p + (RG - 1)*CG + (RG - 1) + (CG - 1)
                ## DRC:
                for(i in 2:(n-1)) for(j in 2:(p-1))
                    if((std=="D")&(RG==i)&(CG==j))
                        npar <- n+p-1 + (RG - 1)*(CG - 1) + (RG - 1) + (CG - 1)
            }
            
            ## Create parameter estimates from lin.a:
            nuab.mat <- matrix(0,n,p)
            if (clustboth == TRUE){
                nuab.mat <- matrix(0,n,p)
                for(rg in 1:RG) for(cg in 1:CG)
                    nuab.mat <- nuab.mat+pi.v[rg]*kappa.v[cg]*lin.a[,,rg,cg]
                nu.s <- mean(nuab.mat)
                alpha.v <- apply(nuab.mat,1,mean)-nu.s
                beta.v <- apply(nuab.mat,2,mean)-nu.s
                gamma.m <- apply(lin.a,c(3,4),mean)-nu.s
            }
            if ((clustrows == TRUE)&(clustcols == FALSE)){
                theta.v <- rep(0,n)
                ## Use lin.a[,,,1], all kappa.v = 1/p
                for(rg in 1:RG) for (j in 1:p)
                    theta.v <- theta.v + pi.v[rg]*1/p*lin.a[,j,rg,1]
                nu.s <- mean(theta.v)
                alpha.v <- theta.v - nu.s
                betagamma.m <- apply(lin.a[,,,1],c(2,3),mean)-nu.s ## p by R
                print(betagamma.m)
                if ((std=="N")|(std=="A")){
                    beta.v <- rep(0,p)
                    gamma.m <- t(betagamma.m)
                }
                if ((std=="B")|(std=="D")){
                    pi.m <- matrix(pi.v,RG,p)  ## RG by p
                    beta.v <- apply(pi.m*t(betagamma.m),2,sum)
                    gamma.m <- t(betagamma.m) - matrix(beta.v,RG,p,byrow=T)
                }
            }
            if ((clustrows == FALSE)&(clustcols == TRUE)){
                theta.v <- rep(0,p)
                ## Use lin.a[,,1,], all pi.v = 1/n
                for(cg in 1:CG) for (i in 1:n)
                    theta.v <- theta.v + kappa.v[cg]*1/n*lin.a[i,,1,cg]
                nu.s <- mean(theta.v)
                beta.v <- theta.v - nu.s
                alphagamma.m <- apply(lin.a[,,1,],c(1,3),mean)-nu.s ## n by C
                print(alphagamma.m)
                if ((std=="N")|(std=="B")){
                    alpha.v <- rep(0,n)
                    gamma.m <- alphagamma.m
                }
                if ((std=="A")|(std=="D")){
                    kappa.m <- matrix(kappa.v,n,CG,byrow=T)  ## n by C
                    alpha.v <- apply(kappa.m*alphagamma.m,1,sum)
                    gamma.m <- alphagamma.m - matrix(alpha.v,n,CG)
                }
            }
            
            ## Name estimates:
            names(alpha.v) <- as.character(1:n)
            names(beta.v) <- as.character(1:p)
            names(pi.v)  <- paste(clustfactnames[1],1:RG,sep="")
            names(kappa.v) <- paste(clustfactnames[2],1:CG,sep="")
            print(gamma.m)
            ## dimnames(gamma.m) <- list(paste(clustfactnames[1],1:RG,sep=""),
            ##                           paste(clustfactnames[2],1:CG,sep=""))
            dimnames(gamma.m) <- list(paste(clustfactnames[1],1:nrow(gamma.m),sep=""),
                                      paste(clustfactnames[2],1:ncol(gamma.m),sep=""))
            if (RG==n) rownames(gamma.m) <- rnames
            if (CG==p) colnames(gamma.m) <- cnames
            dimnames(ppr.m) <- list(rnames,paste(clustfactnames[1],1:RG,sep=""))
            dimnames(ppc.m) <- list(cnames,paste(clustfactnames[2],1:CG,sep=""))
            
            
            ## Save results:
            res.dev <- -2*LL
            aic  <- res.dev+2*npar
            aicc <- NA
            if (Nobs > npar+2)
                aicc <- aic+(2*(npar+1)*(npar+2))/(Nobs-npar-2)
            bic <- -2*LL+npar*log.fn(Nobs)
            icl <- -2*LLc+npar*log.fn(Nobs)
            rclus <- vector("list",RG)
            for (rg in 1:RG)
                rclus[[rg]] <- rnames[ppr.m[,rg]==apply(ppr.m,1,max,na.rm=T)]
            allocated <- c("alloc",rclus[[1]])
            if(RG>1) for (rg in 2:RG){
                if(length(rclus[[rg]])>0){
                    for(kk in length(rclus[[rg]]):1)
                        if(rclus[[rg]][kk] %in% allocated)
                            rclus[[rg]] <- rclus[[rg]][-kk]
                    if (length(rclus[[rg]]>0))
                        allocated <- c(allocated,rclus[[rg]])
                }
            }  
            
            
            
            cclus <- vector("list",CG)
            for (cg in 1:CG)
                cclus[[cg]] <- cnames[ppc.m[,cg]==apply(ppc.m,1,max,na.rm=T)]
            allocated <- c("alloc",cclus[[1]])
            if(CG>1) for (cg in 2:CG){
                if(length(cclus[[cg]])>0){
                    for(kk in length(cclus[[cg]]):1)
                        if(cclus[[cg]][kk] %in% allocated)
                            cclus[[cg]] <- cclus[[cg]][-kk]
                    if (length(cclus[[cg]]>0))
                        allocated <- c(allocated,cclus[[cg]])
                }
            }
            
            ## Create inverse link function:
            if (glmfamily=="bernoulli")
                invlink.fn <- expit.fn
            if (glmfamily=="binomial")
                invlink.fn <- expit.fn
            if (glmfamily=="poisson")
                invlink.fn <- exp
            if (glmfamily=="gaussian")
                invlink.fn <- id.fn
            
            ## Find fitted values and cdf coordinates:
            mu.a <- invlink.fn(lin.a)
            cdf.a <- array(NA,c(n,p,RG,CG))
            cdf.data.m <- matrix(NA,n,p)
            for(i in 1:n) for(j in 1:p){
                if(glmfamily=="poisson")
                    cdf.data.m[i,j] <- (ppois(y.mat[i,j],std.glm.means[i,j])+
                                        ppois(y.mat[i,j]-1,std.glm.means[i,j]))/2
                if(glmfamily=="bernoulli")
                    cdf.data.m[i,j] <- (pbinom(y.mat[i,j],size=1,std.glm.means[i,j])+
                                        pbinom(y.mat[i,j]-1,size=1,std.glm.means[i,j]))/2
                if(glmfamily=="binomial")
                    cdf.data.m[i,j] <- (pbinom(y.mat[i,j],size=ntrials,std.glm.means[i,j])+
                                        pbinom(y.mat[i,j]-1,size=ntrials,
                                               std.glm.means[i,j]))/2
                if(glmfamily=="gaussian"){
                    var.s <- std.glm[[10]]/std.glm[[16]]
                    cdf.data.m[i,j] <- (pnorm(y.mat[i,j],std.glm.means[i,j],
                                              sd=sqrt(var.s))+
                                        pnorm(y.mat[i,j]-1,std.glm.means[i,j],
                                              sd=sqrt(var.s)))/2
                }
            }
            
            ## Construct cdf.group for plotting
            cdf.group.m <- matrix(NA,RG,CG)
            for(rg in 1:RG) for(cg in 1:CG)
                cdf.group.m[rg,cg] <- (t(ppr.m) %*% cdf.data.m %*% ppc.m)[rg,cg]/
                    sum(ppr.m[,rg])/sum(ppc.m[,cg])
            dimnames(cdf.group.m) <- dimnames(gamma.m)
            
            ## If biclustering need 2 more matrices
            cdf.rowgroup.m <- matrix(NA,RG,p)
            cdf.colgroup.m <- matrix(NA,n,CG)
            if((1<RG)&(RG<n)&(1<CG)&(CG<p)){
                for(rg in 1:RG) for(j in 1:p)
                    cdf.rowgroup.m[rg,j] <- sum(ppr.m[,rg]*cdf.data.m[,j])/sum(ppr.m[,rg])
                dimnames(cdf.rowgroup.m) <- list(rownames(cdf.group.m),colnames(y.mat))
                for(i in 1:n) for(cg in 1:CG)
                    cdf.colgroup.m[i,cg] <- sum(ppc.m[,cg]*cdf.data.m[i,])/sum(ppc.m[,cg]) 
                dimnames(cdf.colgroup.m) <- list(rownames(y.mat),colnames(cdf.group.m))
            }
            
            
            ## Construct randomized quantile residuals:
            fits.a <- invlink.fn(lin.a)
            fits.m <- matrix(NA,n,p)
            dimnames(fits.m) <- dimnames(y.mat)
            for (i in 1:n) for (j in 1:p) fits.m[i,j] <-
                (ppr.m %*% fits.a[i,j,,] %*% t(ppc.m))[i,j]
            rqresid.m <- matrix(NA,n,p)
            dimnames(rqresid.m) <- dimnames(y.mat)
            if(glmfamily=="poisson"){
                for (i in 1:n) for (j in 1:p){
                    ## Trap outliers which give NA:
                    if (ppois(y.mat[i,j]-1,fits.m[i,j]) > 0.999999)
                        rqresid.m[i,j] <- 1
                    if (ppois(y.mat[i,j]-1,fits.m[i,j]) <= 0.999999)
                        rqresid.m[i,j] <-
                            runif(1,min = max(0,ppois(y.mat[i,j]-1,fits.m[i,j])),
                                  max = ppois(y.mat[i,j],fits.m[i,j]))
                }
            }
            if(glmfamily=="bernoulli"){
                for (i in 1:n) for (j in 1:p) rqresid.m[i,j] <-
                    runif(1,min = max(0,pbinom(y.mat[i,j]-1,size=1,fits.m[i,j])),
                          max = pbinom(y.mat[i,j],size=1,fits.m[i,j]))
            }
            if(glmfamily=="binomial"){
                for (i in 1:n) for (j in 1:p) rqresid.m[i,j] <-
                    runif(1,min = max(0,pbinom(y.mat[i,j]-1,size=ntrials,fits.m[i,j])),
                          max = pbinom(y.mat[i,j],size=ntrials,fits.m[i,j]))
            }
            if(glmfamily=="gaussian"){
                for (i in 1:n) for (j in 1:p) rqresid.m[i,j] <-
                    runif(1,min = max(0,pnorm(y.mat[i,j]-1,sd=sqrt(var.s),fits.m[i,j])),
                          max = pnorm(y.mat[i,j],sd=sqrt(var.s),fits.m[i,j]))
            }
            
            out1 <- round(c(n,p,LL,res.dev,npar,aic,aicc,bic,icl,LLc,
                            RG,CG,entR,entC),3)
            names(out1) <- c("n","p","Max.ll","Res.Dev.","npar","AIC","AICc",
                             "BIC","ICL","LLc","RG","CG","EntR","EntC")
            if(glmfamily=="binomial")
                out1 <- c(out1,"ntrials"=ntrials)
            
            model.out <- list("info"=out1,
                              "formula"=clustformula,
                              "family"=glmfamily,
                              "data"=data,
                              "std.glm" = std.glm,
                              "this.model.glm" = this.glm,
                              "pi"=pi.v,
                              "kappa"=kappa.v,
                              "nu"=nu.s,
                              "alpha"=alpha.v,
                              "beta"=beta.v,
                              "gamma"=gamma.m,
                              "resid.var"=var.s,
                              "cdf.data"=cdf.data.m,
                              "cdf.group"=cdf.group.m,
                              "cdf.factorgroup1"=cdf.rowgroup.m,
                              "cdf.factorgroup2"=cdf.colgroup.m,
                              "response.fits"=fits.m,
                              "rand.quant.resids"=rqresid.m,
                              "ppr"=ppr.m,
                              "ppc"=ppc.m,
                              "Factor 1 groups"=rclus,
                              "Factor 2 groups"=cclus)
            
        }
        print(names(model.out))
    }
    return(model.out)
}



## E-step - Update posterior probabilities:
## ----------------------------------------

E.step.fn <- function(std,glmfamily,lin.a,y.a,n,p,RG,CG,pi.v,kappa.v,ppr.m,ppc.m, 
                      ntrials=ntrials,var.s=0){
    
    ## Construct mu and logpdf arrays, n by p by RG by CG:
    if (glmfamily=="bernoulli"){
        invlink.fn <- expit.fn
        logpdf.fn <- bernoulli.logpdf
    }
    if (glmfamily=="binomial"){
        invlink.fn <- expit.fn
        logpdf.fn <- binomial.logpdf
    }
    if (glmfamily=="poisson"){
        invlink.fn <- exp
        logpdf.fn <- poisson.logpdf
    }
    if (glmfamily=="gaussian"){
        invlink.fn <- id.fn
        logpdf.fn <- gaussian.logpdf
    }
    
    mu.a <- invlink.fn(lin.a)
    
    if (glmfamily=="bernoulli")
        logpdf.a <- logpdf.fn(mu.a,y.a)
    if (glmfamily=="binomial")
        logpdf.a <- logpdf.fn(mu.a,array(ntrials,c(n,p,RG,CG)),y.a)
    if (glmfamily=="poisson")
        logpdf.a <- logpdf.fn(mu.a,y.a)
    if (glmfamily=="gaussian")
        logpdf.a <- logpdf.fn(mu.a,var.s,y.a)
    
    
    ## -------------------
    ## Rows: Update ppr.m
    ## -------------------
    ## Rule out degenerate case AR1
    if ((RG>1)&(RG<n)&((std!="A")|(CG>1))){
        ## Need to scale to prevent overflow.
        ## Array of adjusters, max, constant over CG.
        ## Construct log.inner array, (i,j,RG,CG)
        ppc.a <- array(ppc.m,c(p,CG,n,RG))
        ppc.a <- aperm(ppc.a,c(3,1,4,2))               ## c(n,p,RG,CG)
        loginner.a <- log.fn(ppc.a) + logpdf.a            ## Aijrc, c(n,p,RG,CG)
        loginner.a[loginner.a==-Inf] <- NA
        ## Array of adjusters, constant over CG.
        B.a <- apply(loginner.a,c(1,2,3),max,na.rm=T)  ## Bijr, c(n,p,RG)
        ## Extend B to extra dimension:
        Bx.a <- array(B.a,c(n,p,RG,CG))              ## Bijrc, c(n,p,RG,CG)
        D.a <- apply(exp(loginner.a-Bx.a),c(1,2,3),sum,na.rm=T)
        E.m <- apply(B.a,c(1,3),sum,na.rm=T)           ## Eir, c(n,RG)
        F.m <- apply(log.fn(D.a),c(1,3),sum,na.rm=T)      ## Fir, c(n,RG)
        ## For each i, find log numerator, adjust to
        ## maximum of zero, then take exp for ppr.m.
        for (i in 1:n){
            lognum.v <- rep(NA,RG)
            for(rg in 1:RG) 
                lognum.v[rg] <- log.fn(pi.v[rg]) + (E.m + F.m)[i,rg]
            adj.lognum <- lognum.v-max(lognum.v,na.rm=T)
            num <- rep(0,RG)
            for(rg in 1:RG) if (!is.na(adj.lognum[rg])){
                if (adj.lognum[rg] > -15)
                    num[rg] <- exp(adj.lognum[rg])
            }
            ppr.m[i,] <- num/sum(num,na.rm=T)
        }
        ppr.m[is.na(ppr.m)] <- 0
    }
    
    
    ## -------------------
    ## Cols: Update ppc.m.
    ## -------------------
    ## Rule out degenerate case B1C
    if ((CG>1)&(CG<p)&((std!="B")|(RG>1))){
        ## Need to scale to prevent overflow.
        ## Array of adjusters, max, constant over RG.
        ## Construct log.inner array, (i,j,RG,CG)
        ppr.a <- array(ppr.m,c(n,RG,p,CG))
        ppr.a <- aperm(ppr.a,c(1,3,2,4))               ## c(n,p,RG,CG)
        loginner.a <- log.fn(ppr.a) + logpdf.a            ## Aijrc, c(n,p,RG,CG)
        loginner.a[loginner.a==-Inf] <- NA
        ## Array of adjusters, constant over RG.
        B.a <- apply(loginner.a,c(1,2,4),max,na.rm=T)
        ## Extend B to extra dimension:
        Bx.a <- array(B.a,c(n,p,CG,RG))
        Bx.a <- aperm(Bx.a,c(1,2,4,3))
        D.a <- apply(exp(loginner.a-Bx.a),c(1,2,4),sum,na.rm=T)
        E.m <- apply(B.a,c(2,3),sum,na.rm=T)
        F.m <- apply(log.fn(D.a),c(2,3),sum,na.rm=T)
        ## For each j, find log numerator, adjust to
        ## maximum of zero, then take exp for ppc.m.
        for (j in 1:p){
            lognum.v <- rep(NA,CG)
            for(cg in 1:CG) 
                lognum.v[cg] <- log.fn(kappa.v[cg])+(E.m + F.m)[j,cg]
            adj.lognum <- lognum.v-max(lognum.v,na.rm=T)
            num <- rep(0,CG)
            for(cg in 1:CG) if (!is.na(adj.lognum[cg])){
                if (adj.lognum[cg] > -15)
                    num[cg] <- exp(adj.lognum[cg])
            }
            ppc.m[j,] <- num/sum(num,na.rm=T)
        }
        ppc.m[is.na(ppc.m)] <- 0
    }
    return(list(ppr.m,ppc.m))
}



## M-step - Update parameter estimates:
## ------------------------------------

## This is the extra long data frame M-step function.

M.step.bi.fn <- function(glmfamily,n,p,
                         RG,CG,ppr.m,ppc.m,
                         clustformula,long.df,std.glm){
    
    pi.v       <- apply(ppr.m,2,mean)
    kappa.v    <- apply(ppc.m,2,mean)
    
    ## Construct weights from zx array n*p*RG*CG
    zx.a <- outer(ppr.m,ppc.m)      # (n*RG*p*CG)
    zx.a <- aperm(zx.a,c(1,3,2,4))  # (n*p*RG*CG)
    zx.v <- c(zx.a)                 # Weights vector for GLM
    
    delt <- 0.000001
    zx.v[zx.v==0] <- delt
    
    ## Extend long data frame to include weights:
    long.df$wts <- zx.v
    
    ## Set some objects to save if needed:
    lin.a <- array(0,c(n,p,RG,CG))
    lc1 <- -1000000
    glm.npar <- 0
    randfailed <- FALSE
    
    ## Fit the glm to the long data:
    if(is.list(try(glm(formula = clustformula,weights=wts,
                       data=long.df,family=glmfamily),silent=T)))
        this.glm <- glm(formula=clustformula,weights=wts,
                        data=long.df,family=glmfamily)
    if(!is.list(try(glm(formula = clustformula,weights=wts,
                        data=long.df,family=glmfamily),silent=T))){
        print("Not finding a solution in this M step")
        this.glm <- std.glm
        randfailed <- TRUE
    }
    
    var.s <- 0
    if (glmfamily=="gaussian")
        var.s <- this.glm$deviance/this.glm$df.residual
    
    if(is.list(this.glm)){
        lc1 <- -this.glm$deviance/2
        ## Calculate full LL for display:
        LLc <- lc1
        LLc <- LLc + (RG>1)*(RG<n)*n*sum(xlogx.fn(pi.v))
        LLc <- LLc + (CG>1)*(CG<p)*p*sum(xlogx.fn(kappa.v))
        entR <- -sum(xlogx.fn(ppr.m))
        entC <- -sum(xlogx.fn(ppc.m))
        LL <- LLc + entR + entC
        if(randfailed==FALSE)
            print(paste("Log likelihood =",round(LL,3)))
        lin.a <- array(this.glm[[9]],c(n,p,RG,CG))
        glm.npar <- sum(!is.na(this.glm[[1]]))
    }
    return(list(lin=lin.a,pi=pi.v,kappa=kappa.v,LL=LL,npar=glm.npar,
                var=var.s,long.df=long.df,this.glm=this.glm))
}


## This is the M-step function for row clustering only.

M.step.rows.fn <- function(glmfamily,n,p,
                           RG,CG,ppr.m,ppc.m,
                           clustformula,long.df,std.glm){
    
    pi.v       <- apply(ppr.m,2,mean)
    kappa.v    <- apply(ppc.m,2,mean)
    
    ## Construct weights from ppr matrix:
    z.a <- array(c(ppr.m),c(n,RG,p))
    z.a <- aperm(z.a,c(1,3,2))  # (n*p*RG)
    z.v <- c(z.a)                 # Weights vector for GLM
    
    delt <- 0.000001
    z.v[z.v==0] <- delt
    
    ## Include weights in long data frame N=n*p*RG:
    long.df$wts <- z.v
    
    ## Set some objects to save if needed:
    lin.a <- array(0,c(n,p,RG,CG))
    lc1 <- -1000000
    glm.npar <- 0
    randfailed <- FALSE
    
    ## Fit the glm to the long data:
    if(is.list(try(glm(formula = clustformula,weights=wts,
                       data=long.df,family=glmfamily),silent=T)))
        this.glm <- glm(formula=clustformula,weights=wts,
                        data=long.df,family=glmfamily)
    if(!is.list(try(glm(formula = clustformula,weights=wts,
                        data=long.df,family=glmfamily),silent=T))){
        print("Not finding a solution in this M step")
        this.glm <- std.glm
        randfailed <- TRUE
    }
    
    var.s <- 0
    if (glmfamily=="gaussian")
        var.s <- this.glm$deviance/this.glm$df.residual
    
    if(is.list(this.glm)){
        lc1 <- -this.glm$deviance/2
        ## Calculate full LL for display:
        LLc <- lc1
        LLc <- LLc + (RG>1)*(RG<n)*n*sum(xlogx.fn(pi.v))
        LLc <- LLc + (CG>1)*(CG<p)*p*sum(xlogx.fn(kappa.v))
        entR <- -sum(xlogx.fn(ppr.m))
        entC <- -sum(xlogx.fn(ppc.m))
        LL <- LLc + entR + entC
        if(randfailed==FALSE)
            print(paste("Log likelihood =",round(LL,3)))
        temp.a <- array(this.glm[[9]],c(n,p,RG))  ## 3D array
        lin.a <- array(temp.a,c(n,p,RG,CG))
        
        glm.npar <- sum(!is.na(this.glm[[1]]))
    }
    return(list(lin=lin.a,pi=pi.v,kappa=kappa.v,LL=LL,npar=glm.npar,
                var=var.s,long.df=long.df,this.glm=this.glm))
}


## This is the M-step function for column clustering only.

M.step.cols.fn <- function(glmfamily,n,p,
                           RG,CG,ppr.m,ppc.m,
                           clustformula,long.df,std.glm){
    
    pi.v       <- apply(ppr.m,2,mean)
    kappa.v    <- apply(ppc.m,2,mean)
    
    ## Construct weights from ppr matrix:
    x.a <- array(c(ppc.m),c(p,CG,n))
    x.a <- aperm(x.a,c(3,1,2))  # (n*p*CG)
    x.v <- c(x.a)                 # Weights vector for GLM
    
    delt <- 0.000001
    x.v[x.v==0] <- delt
    
    ## Include weights in long data frame N=n*p*CG:
    long.df$wts <- x.v
    
    ## Set some objects to save if needed:
    lin.a <- array(0,c(n,p,RG,CG))
    lc1 <- -1000000
    glm.npar <- 0
    randfailed <- FALSE
    
    ## Fit the glm to the long data:
    if(is.list(try(glm(formula = clustformula,weights=wts,
                       data=long.df,family=glmfamily),silent=T)))
        this.glm <- glm(formula=clustformula,weights=wts,
                        data=long.df,family=glmfamily)
    if(!is.list(try(glm(formula = clustformula,weights=wts,
                        data=long.df,family=glmfamily),silent=T))){
        print("Not finding a solution in this M step")
        this.glm <- std.glm
        randfailed <- TRUE
    }
    
    var.s <- 0
    if (glmfamily=="gaussian")
        var.s <- this.glm$deviance/this.glm$df.residual
    
    if(is.list(this.glm)){
        lc1 <- -this.glm$deviance/2
        ## Calculate full LL for display:
        LLc <- lc1
        LLc <- LLc + (RG>1)*(RG<n)*n*sum(xlogx.fn(pi.v))
        LLc <- LLc + (CG>1)*(CG<p)*p*sum(xlogx.fn(kappa.v))
        entR <- -sum(xlogx.fn(ppr.m))
        entC <- -sum(xlogx.fn(ppc.m))
        LL <- LLc + entR + entC
        if(randfailed==FALSE)
            print(paste("Log likelihood =",round(LL,3)))
        lin.a <- array(this.glm[[9]],c(n,p,CG,RG))  ## Full size array
        lin.a <- aperm(lin.a,c(1,2,4,3))
        glm.npar <- sum(!is.na(this.glm[[1]]))
    }
    return(list(lin=lin.a,pi=pi.v,kappa=kappa.v,LL=LL,npar=glm.npar,
                var=var.s,long.df=long.df,this.glm=this.glm))
}




## Double k-Means:
## ---------------

dkm.fn <- function(y.m, RG, CG, Rndstart){
    
    maxiter <- 100
    eps <- 0.0000000001   ## convergence tolerance
    n <- nrow(y.m)
    p <- ncol(y.m)
    ys.mat <- y.m         ## Standardised data matrix
    st <- sum(ys.mat^2)   ## Overall SS:
    
    for (loop in 1:Rndstart){
        ## generate a random partition for variables (cols)
        ppc.ma <- randpart.fn(p,CG)
        ## generate a random partition for units     (rows) 
        ppr.ma <- randpart.fn(n,RG)
        ## generate matrix of means
        su <- apply(ppr.ma,2,sum)  # No. rows in RG
        sv <- apply(ppc.ma,2,sum)  # No. cols in CG
        Ym <- diag(1/su) %*% t(ppr.ma) %*% ys.mat %*% ppc.ma %*% diag(1/sv)
        ## Ym is RG by CG, mean values in each RG and CG
        B <- ppr.ma %*% Ym %*% t(ppc.ma)   # n by p, fitted values
        f0 <- sum(diag((t(B) %*% B)))/st   # explained variance
        it <- 0
        
        ## iteration phase, update the unknown allocations
        while (it<=maxiter){
            it <- it+1
            ## given ys.mat and ppc.ma update ppr.ma
            ppr.ma <- matrix(0,n,RG)
            Ymv <- Ym %*% t(ppc.ma)   # Ymv is RG by p
            for (i in 1:n){
                mindif <- sum((ys.mat[i,]-Ymv[1,])^2)
                posmin <- 1
                for (k in 2:RG){
                    dif <- sum((ys.mat[i,]-Ymv[k,])^2)
                    if (dif < mindif){
                        mindif <- dif
                        posmin <- k
                    } 
                }
                ppr.ma[i,posmin] <- 1
            }
            su <- apply(ppr.ma,2,sum)
            
            ## if a class is empty split the largest class
            while (min(su)==0){
                ## p1 is first col with sum zero
                p1 <- (1:RG)[su==0][1]
                ## p2 is first col with largest sum
                p2 <- (1:RG)[su==max(su)][1]
                ## m is the first row with a 1 in column p2
                m <- (1:n)[ppr.ma[,p2]==1][1]
                ## Interchange 1 and 0 in row m between cols p1 and p2:
                ppr.ma[m,p1] <- 1
                ppr.ma[m,p2] <- 0
                su <- apply(ppr.ma,2,sum)
            } 
            
            ## given ppr.ma and ppc.ma update Ym
            su <- apply(ppr.ma,2,sum)
            Ym <- diag(1/su) %*% t(ppr.ma) %*% ys.mat %*% ppc.ma %*% diag(1/sv)  
            
            ## given ys.mat and ppr.ma update ppc.ma
            ppc.ma <- matrix(0,p,CG)
            Ymu <- ppr.ma %*% Ym
            for (j in 1:p){
                mindif <- sum((ys.mat[,j]-Ymu[,1])^2)
                posmin <- 1
                for (i in 2:CG){
                    dif <- sum((ys.mat[,j]-Ymu[,i])^2)
                    if (dif < mindif){
                        mindif <- dif
                        posmin <- i
                    } 
                }
                ppc.ma[j,posmin] <- 1
            }
            
            sv <- apply(ppc.ma,2,sum)
            
            ## if a class is empty split the largest class
            while (min(sv)==0){
                ## p1 is first col with sum zero
                p1 <- (1:CG)[sv==0][1]
                ## p2 is first col with largest sum
                p2 <- (1:CG)[sv==max(sv)][1]
                ## m is the first row with a 1 in column p2
                m <- (1:p)[ppc.ma[,p2]==1][1]
                ## Interchange 1 and 0 in row m between cols p1 and p2:
                ppc.ma[m,p1] <- 1
                ppc.ma[m,p2] <- 0
                sv <- apply(ppc.ma,2,sum)
            } 
            
            ## given ppr.ma and ppc.ma update Ym
            sv <- apply(ppc.ma,2,sum)
            Ym <- diag(1/su) %*% t(ppr.ma) %*% ys.mat %*% ppc.ma %*% diag(1/sv)
            
            ## Check: is this a better solution?
            B <- ppr.ma %*% Ym %*% t(ppc.ma)
            f <- sum(diag((t(B) %*% B)))/st   
            fdif <- f-f0
            ifelse(fdif > eps,f0 <- f,break)
        }
        if (loop==1){
            Vdkm <- ppc.ma
            Udkm <- ppr.ma
            Ymdkm <- Ym
            fdkm <- f
            loopdkm <- 1
            indkm <- it
            fdifo <- fdif
        }
        if (f > fdkm){
            Vdkm <- ppc.ma
            Udkm <- ppr.ma
            Ymdkm <- Ym
            fdkm <- f
            loopdkm <- loop
            indkm <- it
            fdifo <- fdif
        }
    }
    ## sort clusters of variables per descending order of cardinality
    Vdkm <- Vdkm[,sort.list(-apply(Vdkm,2,sum))]
    ## sort clusters of objects in descending order of cardinality
    Udkm <- Udkm[,sort.list(-apply(Udkm,2,sum))]
    ## Output ppr and ppc:
    list("ppr"=Udkm,"ppc"=Vdkm,"expl.var"=fdkm)
}



randpart.fn <- function(n,c){
    
    ## generates a random partition of n objects in c classes with non-empty classes
    ## n = number of objects
    ## c = number of classes
    pp <- matrix(0,n,c)
    pp[1:c,] <- diag(rep(1,c))  # Ensures non-empty classes
    pp[(c+1):n,1] <- 1
    for (i in (c+1):n)
        pp[i,] <- sample(pp[i,])
    pp <- pp[sample(1:n),]
    pp
}



## Used to stretch out I*J data matrix and up to two covariate matrices
## into data frame with I*J observations.

mat2longdf <- function(y.mat,xr.df=0,xc.df=0){
    ## xr.df is a data frame of row-based covariates (eg. site factors)
    ## xc.df is a data frame of column-based covariates (eg. species traits)
    ## Lengthen the row covariates data frame:
    if(is.data.frame(xr.df)){
        xr.longdf <- xr.df
        for (i in 1:(ncol(y.mat)-1))
            xr.longdf <- rbind(xr.longdf,xr.df)
    }
    ## For xc, construct in wrong order, then rearrange:
    if(is.data.frame(xr.df)){
        xc.longdf <- xc.df
        for (i in 1:(nrow(y.mat)-1))
            xc.longdf <- rbind(xc.longdf,xc.df)
        ## Rearrangement preserves factors, numeric
        for (j in 1:ncol(xc.longdf))
            xc.longdf[,j] <- rep(xc.df[,j],each=nrow(y.mat))
    }
    ## Build long data frame:
    my.df <- data.frame(Y = c(y.mat),
                        ROW = gl(nrow(y.mat),1,nrow(y.mat)*ncol(y.mat)),
                        COL = gl(ncol(y.mat),nrow(y.mat),nrow(y.mat)*ncol(y.mat)))
    if(is.data.frame(xr.df))
        my.df <- cbind(my.df,xr.longdf)
    if(is.data.frame(xc.df))
        my.df <- cbind(my.df,xc.longdf)
    rownames(my.df) <- as.character(1:nrow(my.df))
    ## Return long data frame:
    return(my.df)
}








## IC table:
## ---------

ICtable.fn <- function(model.list){
    
    IC.df <- data.frame("Model"=names(model.list),
                        ## "formula"=rep(NA,length(model.list)),
                        "RG"=rep(NA,length(model.list)),
                        "CG"=rep(NA,length(model.list)),
                        "LL"=rep(NA,length(model.list)),
                        "npar"=rep(NA,length(model.list)),
                        "AIC"=rep(NA,length(model.list)),
                        "AICc"=rep(NA,length(model.list)),
                        "BIC"=rep(NA,length(model.list)))
    
    for(i in 1:nrow(IC.df)){
        ## IC.df$formula[i] <- model.list[[i]][[2]]
        IC.df$RG[i] <- model.list[[i]][[1]][11]
        IC.df$CG[i] <- model.list[[i]][[1]][12]
        IC.df$LL[i] <- model.list[[i]][[1]][3]
        IC.df$npar[i] <- model.list[[i]][[1]][5]
        IC.df$AIC[i] <- model.list[[i]][[1]][6]
        IC.df$AICc[i] <- model.list[[i]][[1]][7]
        IC.df$BIC[i] <- model.list[[i]][[1]][8]
    }
    
    ## IC.df$formula <- as.character(IC.df$formula)
    IC.df$deltaAIC <- IC.df$AIC-min(IC.df$AIC,na.rm=T)
    IC.df$deltaAICc <- IC.df$AICc-min(IC.df$AICc,na.rm=T)
    IC.df$deltaBIC <- IC.df$BIC-min(IC.df$BIC,na.rm=T)
    IC.df
}




## Plot functions:
## ---------------

ICplot.fn <- function(plot.list,ICtype,colourvector=NULL,
                      lowlabelposition=1,highlabelposition=1,
                      plottitle=NULL){
    
    plot.models <- ICtable.fn(plot.list)
    n <- plot.list[[1]][[1]][1]
    p <- plot.list[[1]][[1]][2]
    
    if(is.null(colourvector))
        uniquecolours <- rep("black",50)
    if(!is.null(colourvector))
        uniquecolours <- rep(colourvector,50)
    
    ## uniquemodeltypes <- unique(plot.models$type)
    for(i in 1:nrow(plot.models))
        plot.models$colour[i] <- uniquecolours[i]
    
    if(is.null(plottitle))
        mainheading <- paste("Log likelihood by npar, with",ICtype,"contours",sep=" ")
    if(!is.null(plottitle))
        mainheading <- plottitle
    
    LL.range <- range(plot.models$LL)
    LL.splits <- c(LL.range[2]-(-LL.range[1]+LL.range[2])/8,
                   LL.range[2]-(-LL.range[1]+LL.range[2])/4,
                   (-LL.range[1]+LL.range[2])/2+LL.range[1],
                   (-LL.range[1]+LL.range[2])/4+LL.range[1],
                   (-LL.range[1]+LL.range[2])/8+LL.range[1])
    npar.range <- range(plot.models$npar)
    npar.splits <- c(npar.range[1]+(npar.range[2]-npar.range[1])/10,
                     npar.range[1]+(npar.range[2]-npar.range[1])/6,
                     npar.range[2]-(npar.range[2]-npar.range[1])/6,
                     npar.range[2]-(npar.range[2]-npar.range[1])/10,
                     (npar.range[2]-npar.range[1])/10+npar.range[2])
    
    
    ## Create the plot
    plot(plot.models$npar,plot.models$LL,type="n",
         col=plot.models$colour,xlab="No. parameters",ylab="log Likelihood",
         cex=0.8,main=mainheading)
    
    
    if(ICtype=="AIC"){
        criterion.range <- range(plot.models$AIC)
        thisIC.fn <- AIC.fn
    }
    if(ICtype=="AICc"){
        criterion.range <- range(plot.models$AICc)
        thisIC.fn <- AICc.fn
    }
    if(ICtype=="BIC"){
        criterion.range <- range(plot.models$BIC)
        thisIC.fn <- BIC.fn
    }
    
    criterion.levels <- seq(from=criterion.range[1],to=criterion.range[2],length.out=6)
    xx <- seq(npar.range[1],npar.range[2],length.out=100)
    yy <- seq(LL.range[1],LL.range[2],length.out=100)
    zz <- matrix(NA,length(xx),length(yy))
    for(i in 1:length(xx)) for(j in 1:length(yy))
        zz[i,j] <- thisIC.fn(xx[i],yy[j],n,p)
    
    ## Draw IC contours
    contour(xx,yy,zz,add=T,levels=round(criterion.levels,2),lty=5,col="grey")
    
    ## Choose locations of the labels
    if(lowlabelposition==1){
        if(ICtype=="AIC")
            text(npar.splits[4],LL.splits[5],"High AIC",cex=1.0,col="grey")
        if(ICtype=="AICc")
            text(npar.splits[4],LL.splits[5],"High AICc",cex=1.0,col="grey")
        if(ICtype=="BIC")
            text(npar.splits[4],LL.splits[5],"High BIC",cex=1.0,col="grey")
    }
    if(lowlabelposition==2){
        if(ICtype=="AIC")
            text(npar.splits[3],LL.splits[4],"High AIC",cex=1.0,col="grey")
        if(ICtype=="AICc")
            text(npar.splits[3],LL.splits[4],"High AICc",cex=1.0,col="grey")
        if(ICtype=="BIC")
            text(npar.splits[3],LL.splits[4],"High BIC",cex=1.0,col="grey")
    }
    if(highlabelposition==1){
        if(ICtype=="AIC")
            text(npar.splits[1],LL.splits[1],"Low AIC",cex=1.0,col="grey")
        if(ICtype=="AICc")
            text(npar.splits[1],LL.splits[1],"Low AICc",cex=1.0,col="grey")
        if(ICtype=="BIC")
            text(npar.splits[1],LL.splits[1],"Low BIC",cex=1.0,col="grey")
    }
    if(highlabelposition==2){
        if(ICtype=="AIC")
            text(npar.splits[2],LL.splits[2],"Low AIC",cex=1.0,col="grey")
        if(ICtype=="AICc")
            text(npar.splits[2],LL.splits[2],"Low AICc",cex=1.0,col="grey")
        if(ICtype=="BIC")
            text(npar.splits[2],LL.splits[2],"Low BIC",cex=1.0,col="grey")
    }
    
    text(plot.models$npar,plot.models$LL,as.character(plot.models$Model),cex=1.0,
         col=plot.models$colour)
}




ordplot.fn <- function(model.in,ordtype,rlabels=NULL,clabels=NULL,
                       xlabel=NULL,ylabel=NULL,plottitle=NULL){
    
    n <- model.in[[1]][1]
    p <- model.in[[1]][2]
    RG <- model.in[[1]][11]
    CG <- model.in[[1]][12]
    cdf.group.m <- model.in$cdf.group
    
    plotRG <- RG
    if(plotRG==n)
        plotRG <- "n"
    plotCG <- CG
    if(plotCG==p)
        plotCG <- "p"
    
    if(ordtype=="rows"){
        if(RG<n)
            stop("For an ordination of the rows, a column clustered model with all 
                 rows separate is needed.")
        if(is.null(rlabels))
            plottext <- rownames(cdf.group.m)
        if(!is.null(rlabels))
            plottext <- rlabels
        if(CG==2)
            points2D <- cdf.group.m
        if(CG==3){
            x <- cdf.group.m - min(cdf.group.m)
            s <- rowSums(x)
            x <- x/s
            top <- sqrt(3)/2
            xlim <- c(-0.03, 1.03)
            ylim <- c(-1, top)
            xp <- x[, 2] + x[, 3]/2
            yp <- x[, 3] * top
            points2D <- cbind(xp,yp)
        }
    }
    
    if(ordtype=="cols"){
        if(CG<p)
            stop("For an ordination of the columns, a row clustered model with all 
                 columns separate is needed.")
        if(is.null(clabels))
            plottext <- colnames(cdf.group.m)
        if(!is.null(clabels))
            plottext <- clabels
        if(RG==2)
            points2D <- t(cdf.group.m)
        if(RG==3){
            x <- t(cdf.group.m) - min(cdf.group.m)
            s <- rowSums(x)
            x <- x/s
            top <- sqrt(3)/2
            xlim <- c(-0.03, 1.03)
            ylim <- c(-1, top)
            xp <- x[, 2] + x[, 3]/2
            yp <- x[, 3] * top
            points2D <- cbind(xp,yp)
        }
    }
    
    if(is.null(plottitle))
        mainheading <- paste("Ordination of ",ordtype)
    if(!is.null(plottitle))
        mainheading <- plottitle
    
    if(is.null(xlabel))
        xlabel <- "Dimension 1"
    if(is.null(ylabel))
        ylabel <- "Dimension 2"
    
    ## Create the plot
    plot(points2D,type="n",xlab=xlabel,ylab=ylabel,main=mainheading,asp=1)
    text(points2D,plottext)
    
}



profileplot.fn <- function(model.in,profiletype,mysort=NULL,sorttype=NULL,
                           sortgroup=NULL,plottitle=NULL,xlabel=NULL,
                           colourvector=NULL,linetype=NULL,linewidth=1){
    
    n <- model.in[[1]][1]
    p <- model.in[[1]][2]
    RG <- model.in[[1]][11]
    CG <- model.in[[1]][12]
    cdf.group.m <- model.in$cdf.group
    
    plotRG <- RG
    if(plotRG==n)
        plotRG <- "n"
    plotCG <- CG
    if(plotCG==p)
        plotCG <- "p"
    
    if(is.null(colourvector))
        uniquecolours <- c("cornflowerblue","orange2","palegreen3",
                           "palevioletred1","goldenrod4","cadetblue4",
                           "mediumorchid3","yellow3","indianred3")
    if(!is.null(colourvector))
        uniquecolours <- colourvector
    if(is.null(linetype))
        linetypes <- rep(1,max(c(RG,CG)))
    if(!is.null(linetype))
        linetypes <- linetype
    
    if(profiletype=="rowgroups"){
        if(is.null(xlabel))
            xlabel <- "Column"
        if(CG<p)
            stop("For a profile plot of the row groups, a row clustered model with all 
                 columns separate is needed.")
        if(is.null(sorttype))
            cols.sort <- 1:p
        if(!is.null(sorttype)){
            if(sorttype=="mean")
                cols.sort <- (1:p)[sort.list(apply(cdf.group.m,2,mean))]
            if(sorttype=="group"){
                if(is.null(sortgroup))
                    stop("Which group do you want to sort by?")
                cols.sort <- (1:p)[sort.list(cdf.group.m[sortgroup,])]
            }
            if(sorttype=="user.specified"){
                if(is.null(mysort))
                    stop("Need to specify order in mysort")
                if(length(mysort)!=p)
                    stop("Need to specify mysort of length p")
                cols.sort <- mysort
            }
            if(sorttype=="travellingsalesman"){
                ## Set up the distance travelled with cols. 1:p:
                tp <- 0
                sc <- 1:p
                for(rg in 1:RG) tp <-
                    tp + sum(abs(diff(cdf.group.m[RG,sc])))
                total.path <- tp    ## Best total path so far
                cols.sort <- sc
                ## Find furthest apart columns:
                dist.mat <- as.matrix(dist(t(cdf.group.m),method="manhattan"))
                endpts <- which(dist.mat==max(dist.mat),arr.ind=T)
                start.col <- endpts[1,1]
                end.col   <- endpts[1,2]
                other.cols <- (1:p)[-c(start.col,end.col)]
                ## Try random orderings of internal columns:
                for (i in 1:100000){
                    sc <- c(start.col,sample(other.cols),end.col)
                    tp <- 0
                    for(rg in 1:RG) tp <-
                        tp + sum(abs(diff(cdf.group.m[RG,sc])))
                    if (tp < total.path){
                        print("new best")
                        print(sc)
                        print(tp)
                        total.path <- tp
                        cols.sort <- sc
                    }
                }
            }
        }
        
        if(is.null(plottitle))
            mainheading <- paste("Profile plot of ",RG," row groups",sep="")
        if(!is.null(plottitle))
            mainheading <- plottitle
        
        ## Create the plot
        plot(1:p,cdf.group.m[1,cols.sort],type="l",col=uniquecolours[1],
             axes=F,ylim=range(cdf.group.m),xlab=xlabel,ylab="CDF",
             main=mainheading,lty=linetypes[1],lwd=linewidth)
        axis(1,1:p,as.character(1:p)[cols.sort])
        axis(2)
        box()
        for(k in 2:RG)
            lines(1:p,cdf.group.m[k,cols.sort],col=uniquecolours[k],
                  lty=linetypes[k],lwd=linewidth)
    }
    
    if(profiletype=="colgroups"){
        if(is.null(xlabel))
            xlabel <- "Row"
        if(RG<n)
            stop("For a profile plot of the column groups, a column clustered model 
                 with all rows separate is needed.")
        if(is.null(sorttype))
            rows.sort <- 1:n
        if(!is.null(sorttype)){
            if(sorttype=="mean")
                rows.sort <- (1:n)[sort.list(apply(cdf.group.m,1,mean))]
            if(sorttype=="group"){
                if(is.null(sortgroup))
                    stop("Which group do you want to sort by?")
                rows.sort <- (1:n)[sort.list(cdf.group.m[,sortgroup])]
            }
            if(sorttype=="user.specified"){
                if(is.null(mysort))
                    stop("Need to specify order in mysort")
                if(length(mysort)!=n)
                    stop("Need to specify mysort of length n")
                rows.sort <- mysort
            }
            if(sorttype=="travellingsalesman"){
                ## Set up the distance travelled with cols. 1:p:
                tp <- 0
                sr <- 1:n
                for(cg in 1:CG) tp <-
                    tp + sum(abs(diff(cdf.group.m[sr,CG])))
                total.path <- tp    ## Best total path so far
                rows.sort <- sr
                ## Find furthest apart columns:
                dist.mat <- as.matrix(dist(cdf.group.m,method="manhattan"))
                endpts <- which(dist.mat==max(dist.mat),arr.ind=T)
                start.row <- endpts[1,1]
                end.row   <- endpts[1,2]
                other.rows <- (1:n)[-c(start.row,end.row)]
                ## Try random orderings of internal columns:
                for (i in 1:100000){
                    sr <- c(start.row,sample(other.rows),end.row)
                    tp <- 0
                    for(cg in 1:CG) tp <-
                        tp + sum(abs(diff(cdf.group.m[sr,CG])))
                    if (tp < total.path){
                        print("new best")
                        print(sr)
                        print(tp)
                        total.path <- tp
                        rows.sort <- sr
                    }
                }
            }
        }
        
        if(is.null(plottitle))
            mainheading <- paste("Profile plot of ",CG," column groups",sep="")
        if(!is.null(plottitle))
            mainheading <- plottitle
        
        ## Create the plot
        plot(1:n,cdf.group.m[rows.sort,1],type="l",col=uniquecolours[1],
             axes=F,ylim=range(cdf.group.m),xlab=xlabel,ylab="CDF",
             main=mainheading,lty=linetypes[1],lwd=linewidth)
        axis(1,1:n,as.character(1:n)[rows.sort])
        axis(2)
        box()
        for(k in 2:CG)
            lines(1:n,cdf.group.m[rows.sort,k],col=uniquecolours[k],
                  lty=linetypes[k],lwd=linewidth)
    }
}



biplot.fn <- function(model.in,data,colourvector=NULL,plottitle=NULL,
                      myratio=1,centrepoints=T){
    
    n <- model.in[[1]][1]
    p <- model.in[[1]][2]
    RG <- model.in[[1]][11]
    CG <- model.in[[1]][12]
    cdf.group.m <- model.in$cdf.group
    cdf.rowgroup.m <- model.in$cdf.factorgroup1
    cdf.colgroup.m <- model.in$cdf.factorgroup2
    
    gamma.m <- model.in$gamma
    
    plotRG <- RG
    if(plotRG==n)
        plotRG <- "n"
    plotCG <- CG
    if(plotCG==p)
        plotCG <- "p"
    if((CG!=2)|(RG!=2))
        stop("For a 2D biplot, you need two row groups and two column groups.")
    if(is.null(colourvector))
        uniquecolours <- c("cornflowerblue","orange2")
    if(!is.null(colourvector))
        uniquecolours <- colourvector
    if(is.null(plottitle))
        mainheading <- paste("Mixture-based biplot, ",plotRG,"*",plotCG," groups",sep="")
    if(!is.null(plottitle))
        mainheading <- plottitle
    
    SVDCDF <- svd(cdf.group.m)
    U.mat  <- SVDCDF$u
    V.mat  <- SVDCDF$v
    sqrtS  <- diag(sqrt(SVDCDF$d))
    G.mat  <- U.mat %*% sqrtS
    H.mat  <- V.mat %*% sqrtS
    Gn.mat <- cdf.colgroup.m %*% V.mat %*% solve(sqrtS) 
    Hp.mat <- t(solve(sqrtS) %*% t(U.mat) %*% cdf.rowgroup.m)
    GH.mat <- rbind(G.mat,H.mat,Gn.mat,Hp.mat)[,1:2]
    
    rnames <- rownames(y.mat)
    cnames <- colnames(y.mat)
    
    ## Create the plot
    eqscplot(GH.mat,axes=F,type="n",xlab="Dimension 1",ylab="Dimension 2",
             pch=3,cex=2,main=mainheading,ratio=myratio)
    axis(1)
    axis(2)
    box()
    if(centrepoints==T){
        points(GH.mat[1:2,1],GH.mat[1:2,2],pch=16,col=uniquecolours[1])
        points(GH.mat[3:4,1],GH.mat[3:4,2],pch=16,col=uniquecolours[2])
    }
    text(GH.mat[5:(4+n),1],GH.mat[5:(4+n),2],rnames,col=uniquecolours[1])
    text(GH.mat[(5+n):(4+n+p),1],GH.mat[(5+n):(4+n+p),2],cnames,col=uniquecolours[2])
}

