## clustglm_run.R


### ------------------------------------------------------
###               Prepare workspace
### ------------------------------------------------------


## Ensure that the correct working directory is selected using the setwd() command

## Clear the workspace if required:

rm(list=ls())





### ------------------------------------------------------
###               Run Initial Setup 
### ------------------------------------------------------


## Read in the required functions and libraries:

source("clustglm_fun.R")
library(cluster)
library(MASS)





### ------------------------------------------------------
###               Load and tidy Dataset
### ------------------------------------------------------


## The example dataset provided is the Whittaker shrubs dataset.

## Load the dataset and any covariates:

whitshrubs.df <- read.csv("whitshrubsfull.csv",T)
whitshrubs_sitecov.df <- read.csv("whitshrubsfull_sitecov.csv",T)
whitshrubs_speciescov.df <- read.csv("whitshrubsfull_speciescov.csv",T)

## Convert the dataset to a matrix:

whitshrubs.mat <- as.matrix(whitshrubs.df[,-1])
whitshrubs.mat <- t(whitshrubs.mat)


## Save it as "y.mat":

y.mat <- whitshrubs.mat   # 10 sites by 25 spp.

## If not already present, you can give it row and column names:

rownames(y.mat) <- c(as.character(1:10))
colnames(y.mat) <- c(as.character(1:25))


## Create a long dataframe with both data matrix and covariate information:

glm.df <- mat2longdf(y.mat,xr.df=whitshrubs_sitecov.df,xc.df=whitshrubs_speciescov.df)


## Note: The Whittaker shrubs dataset should be modelled using a Poisson distribution, 
## but can also be modelled using a Binomial or Gaussian distribution.
## To model it using a Bernoulli distribution, you will need to use the command:
## y.mat <- sign(y.mat)
## For a binomial distribution "ntrials" must be specified, greater than or equal to
## the largest count in y.mat.



### ------------------------------------------------------
###               Run the Models 
### ------------------------------------------------------


## Now that the data is loaded, you can run the desired models. A selection of the 
## different options is presented below. It is important to save the output of the 
## models so it can be viewed later.

## Generalised linear models:

N11.out <- clustglm(glmformula = Y ~ 1, glmfamily = "poisson", data = glm.df)

A11.out <- clustglm(glmformula = Y ~ ROW, glmfamily = "poisson", data = glm.df)

B11.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df)

D11.out <- clustglm(glmformula = Y ~ ROW + COL, glmfamily = "poisson", data = glm.df)

## Looking at N11.out[[1]], A11.out[[1]], B11.out[[1]] and D11.out[[1]], we note that
## the B model (allowing for a main effect of columns but not rows) has the lowest
## AIC and BIC. We pursue clustered models of standardization type "B".

## Row clustered models, B standardisation:

B2p.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0, 
                    numbersclusters=c(2,25), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COL)

B3p.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(3,25), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COL)

B4p.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(4,25), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COL)

B5p.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(5,25), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COL)


## Alternatively, we may decide the D standardization is more appropriate:

D2p.out <- clustglm(glmformula = Y ~ ROW + COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0, 
                    numbersclusters=c(2,25), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ ROW + COL + ROWclust:COL)

## Similarly D3p, D4p, etc.


## Column clustered models:

## Column clustered models, B standardization:

Bn2.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(10,2), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROW:COLclust)

Bn3.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(10,3), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROW:COLclust)

Bn4.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(10,4), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROW:COLclust)

Bn5.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(10,5), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROW:COLclust)


## Column clustered models, D standardization:

Dn2.out <- clustglm(glmformula = Y ~ ROW + COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=0,
                    numbersclusters=c(10,2), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ ROW + COL + ROW:COLclust)

## Similarly Dn3, Dn4 etc.



## Biclustered models, B standardization:

B22.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=5,
                    numbersclusters=c(2,2), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COLclust)

B33.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=5,
                    numbersclusters=c(3,3), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COLclust)

B44.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=5,
                    numbersclusters=c(4,4), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COLclust)

B55.out <- clustglm(glmformula = Y ~ COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=5,
                    numbersclusters=c(5,5), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ COL + ROWclust:COLclust)


## Biclustered models, D standardization:

D22.out <- clustglm(glmformula = Y ~ ROW + COL, glmfamily = "poisson", data = glm.df,
                    clustfact = c("ROW","COL"), randstarts=5,
                    numbersclusters=c(2,2), clustfactnames=c("ROWclust","COLclust"),
                    clustformula = Y ~ ROW + COL + ROWclust:COLclust)

## Similarly D33, D44 etc.



## Substitute Plant Type for Species in 4*p model
Plant.Type4p.out <- clustglm(glmformula = Y ~ Plant.Type, glmfamily = "poisson", 
                             data = glm.df, clustfact = c("ROW","COL"), 
                             randstarts=5, numbersclusters=c(4,25), 
                             clustfactnames=c("ROWclust","COLclust"),
                             clustformula = Y ~ Plant.Type + ROWclust:Plant.Type)

## Include Terrain for M.E. of Site but keep Species in 4*p model
Terrain4p.out <- clustglm(glmformula = Y ~ Terrain + COL, glmfamily = "poisson", 
                                  data = glm.df, clustfact = c("ROW","COL"), 
                                  randstarts=5, numbersclusters=c(4,25), 
                                  clustfactnames=c("ROWclust","COLclust"),
                                  clustformula = Y ~ Terrain + COL + ROWclust:COL)





### -------------------------------------------------------------
###               Summarise and save the output
### -------------------------------------------------------------


## Each model output has a wide range of information about the model. The available 
## information can be viewed using the following command:

names(B33.out)


## Examples of how to examine a particular part of information:

B33.out$info
B33.out$pp.ROW


## To select the best model from all of the models run using an information 
## critereon, a table can be constructed for easy comparisons. 

## First create a list that includes all of the models that were run:

model.list <- list("N11"=N11.out,"A11"=A11.out,"B11"=B11.out,"D11"=D11.out,
                   "B2p"=B2p.out,"B3p"=B3p.out,"B4p"=B4p.out,"B5p"=B5p.out,
                   "Bn2"=Bn2.out,"Bn3"=Bn3.out,"Bn4"=Bn4.out,"Bn5"=Bn5.out,
                   "B22"=B22.out,"B33"=B33.out,"B44"=B44.out,"B55"=B55.out,
                   "PT4p"=Plant.Type4p.out,"T4p"=Terrain4p.out)

## Then construct the IC table:

ICtable <- ICtable.fn(model.list)
ICtable
ICtable[,-(6:8)]



## Save the model outputs and ICtable:

model.list$ICtable <- ICtable.fn(model.list)
Whittaker.out <- model.list

sink("Whittaker.out")
Whittaker.out
sink()






### -------------------------------------------------------------
###               Create Plots
### -------------------------------------------------------------


## A wide range of optional arguments are available for the following plots. 
## Only a few are used in the examples below, to see the the full range of 
## options please see attached documentation.


## IC plot:
## --------

## An infomation critereon plot can be constructed to visually examine how different 
## models compare using a selected information critereon. 

## First create a list of the models to be plotted:

plot.list <- list("N"=N11.out,"A"=A11.out,"B"=B11.out,"D"=D11.out,
                  "2J"=B2p.out,"3J"=B3p.out,"4J"=B4p.out,"5J"=B5p.out,
                  "I2"=Bn2.out,"I3"=Bn3.out,"I4"=Bn4.out,"I5"=Bn5.out,
                  "22"=B22.out,"33"=B33.out,"44"=B44.out,"55"=B55.out)

## Then construct the plot: 

## If no colour vector is specified, all points will be plotted in black
colours <- c("cornflowerblue","cornflowerblue","cornflowerblue","cornflowerblue",
             "orange2","orange2","orange2","orange2",
             "palegreen3","palegreen3","palegreen3","palegreen3",
             "palevioletred1","palevioletred1","palevioletred1","palevioletred1",
             "goldenrod4","goldenrod4","goldenrod4","goldenrod4")

ICplot.fn(plot.list, ICtype="AIC", colourvector=colours, highlabelposition=1,lowlabelposition=2,
          plottitle="Log likelihood by npar, with AIC contours")
ICplot.fn(plot.list, ICtype="AICc", colourvector=colours, highlabelposition=1,lowlabelposition=2,
          plottitle="Log likelihood by npar, with AICc contours")
ICplot.fn(plot.list, ICtype="BIC", colourvector=colours, highlabelposition=1,lowlabelposition=2,
          plottitle="Log likelihood by npar, with BIC contours")

## Note that if you set highlabelposition = 0 that will remove the high corner label.  







## Ordination plot:
## ----------------

## An ordination plot can be constructed to visually examine the groupings of 
## either the rows or the columns. 

## Note: An ordination plot must have either all rows separate and 2 column 
## clusters or all columns separate and 2 row clusters.

## Try the B2p.out and Bn2.out models that were run earlier

## Construct the plots: 

ordplot.fn(model.in=B2p.out, ordtype="cols", plottitle="Ordination of cols, model B2p")

ordplot.fn(model.in=Bn2.out, ordtype="rows", plottitle="Ordination of rows, model Bn2")

## Can optionally name the points using either the rlabels or clabels argument
ordplot.fn(model.in=Bn2.out, ordtype="rows",
           rlabels=c("A","B","C","D","E","F","G","H","I","J"),
           plottitle="Ordination of rows, model Bn2")

  

## Profile plot:
## -------------

## A profile plot can be constructed to visually examine the groupings of 
## either the rows or the columns. 

## Note: A profile plot must have either all rows separate and columns 
## clustered or all columns separate and rows clustered.


## Then construct the plots:

profileplot.fn(model.in=B3p.out, profiletype="rowgroups", sorttype="group", sortgroup=1)

profileplot.fn(model.in=Bn3.out, profiletype="colgroups", sorttype="travellingsalesman",
               linewidth=2, linetype=c(1,2,3))

  

## Biplot:
## -------

## A biplot can be constructed to visually examine the row and column 
## groupings simultaneously. 

## A biplot must have 2 row groups and 2 column groups.

biplot.fn(model.in=B22.out, data=y.mat, myratio=0.1, centrepoints=F)










